# -*- coding: utf-8 -*-
"""
Integration tests for the zookeeper states
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import logging

# Import Salt Libs
import salt.utils.path
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest, slowTest
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Testing Libs
from tests.support.unit import skipIf

try:
    import kazoo  # pylint: disable=import-error,unused-import

    HAS_KAZOO = True
except ImportError:
    HAS_KAZOO = False

log = logging.getLogger(__name__)


@destructiveTest
@skipIf(not salt.utils.path.which("dockerd"), "Docker not installed")
@skipIf(not HAS_KAZOO, "kazoo python library not installed")
class ZookeeperTestCase(ModuleCase, SaltReturnAssertsMixin):
    """
    Test zookeeper states
    """

    @classmethod
    def setUpClass(cls):
        cls.container_name = "zookeeper_salt"

    def setUp(self):
        self.run_state("docker_image.present", name="zookeeper")
        self.run_state(
            "docker_container.running",
            name=self.container_name,
            image="zookeeper",
            port_bindings="2181:2181",
        )

    def tearDown(self):
        self.run_state("docker_container.stopped", name=self.container_name)
        self.run_state("docker_container.absent", name=self.container_name)
        self.run_state("docker_image.absent", name="docker.io/zookeeper", force=True)

    @slowTest
    def test_zookeeper_present(self):
        ret = self.run_state(
            "zookeeper.present", name="/test/name", value="testuser", makepath=True,
        )
        self.assertSaltTrueReturn(ret)

        ret = self.run_state(
            "zookeeper.present",
            name="/test/name",
            value="daniel",
            acls=[
                {
                    "username": "daniel",
                    "password": "test",
                    "read": True,
                    "admin": True,
                    "write": True,
                },
                {"username": "testuser", "password": "test", "read": True},
            ],
            profile="prod",
        )
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_zookeeper_absent(self):
        self.run_state(
            "zookeeper.present", name="/test/name", value="testuser", makepath=True,
        )
        ret = self.run_state("zookeeper.absent", name="/test/name",)
        self.assertSaltTrueReturn(ret)
        self.assertTrue(
            bool(ret["zookeeper_|-/test/name_|-/test/name_|-absent"]["changes"])
        )
        ret = self.run_state("zookeeper.absent", name="/test/name",)
        self.assertFalse(
            bool(ret["zookeeper_|-/test/name_|-/test/name_|-absent"]["changes"])
        )

    @slowTest
    def test_zookeeper_acls(self):
        ret = self.run_state(
            "zookeeper.acls",
            name="/test/name",
            acls=[
                {
                    "username": "daniel",
                    "password": "test",
                    "read": True,
                    "admin": True,
                    "write": True,
                },
                {"username": "testuser", "password": "test", "read": True},
            ],
        )
        self.assertSaltFalseReturn(ret)

        ret = self.run_state(
            "zookeeper.present", name="/test/name", value="testuser", makepath=True,
        )

        ret = self.run_state(
            "zookeeper.acls",
            name="/test/name",
            acls=[
                {
                    "username": "daniel",
                    "password": "test",
                    "read": True,
                    "admin": True,
                    "write": True,
                },
                {"username": "testuser", "password": "test", "read": True},
            ],
        )
        self.assertSaltTrueReturn(ret)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import salt.utils.platform
from tests.support.case import SSHCase
from tests.support.helpers import slowTest
from tests.support.unit import skipIf


@skipIf(salt.utils.platform.is_windows(), "salt-ssh not available on Windows")
class SSHGrainsTest(SSHCase):
    """
    testing grains with salt-ssh
    """

    @slowTest
    def test_grains_items(self):
        """
        test grains.items with salt-ssh
        """
        ret = self.run_function("grains.items")
        grain = "Linux"
        if salt.utils.platform.is_darwin():
            grain = "Darwin"
        if salt.utils.platform.is_aix():
            grain = "AIX"
        self.assertEqual(ret["kernel"], grain)
        self.assertTrue(isinstance(ret, dict))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Mike Place <mp@saltstack.com>
"""
from __future__ import absolute_import

import copy
import os

import salt.ext.tornado
import salt.ext.tornado.testing
import salt.minion
import salt.syspaths
import salt.utils.crypt
import salt.utils.event as event
import salt.utils.process
from salt.exceptions import SaltMasterUnresolvableError, SaltSystemExit
from salt.ext.six.moves import range
from tests.support.helpers import skip_if_not_root, slowTest
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
    def setUp(self):
        self.opts = {}
        self.addCleanup(delattr, self, "opts")

    def test_invalid_master_address(self):
        with patch.dict(
            self.opts,
            {
                "ipv6": False,
                "master": float("127.0"),
                "master_port": "4555",
                "retry_dns": False,
            },
        ):
            self.assertRaises(SaltSystemExit, salt.minion.resolve_dns, self.opts)

    def test_source_int_name_local(self):
        """
        test when file_client local and
        source_interface_name is set
        """
        interfaces = {
            "bond0.1234": {
                "hwaddr": "01:01:01:d0:d0:d0",
                "up": True,
                "inet": [
                    {
                        "broadcast": "111.1.111.255",
                        "netmask": "111.1.0.0",
                        "label": "bond0",
                        "address": "111.1.0.1",
                    }
                ],
            }
        }
        with patch.dict(
            self.opts,
            {
                "ipv6": False,
                "master": "127.0.0.1",
                "master_port": "4555",
                "file_client": "local",
                "source_interface_name": "bond0.1234",
                "source_ret_port": 49017,
                "source_publish_port": 49018,
            },
        ), patch("salt.utils.network.interfaces", MagicMock(return_value=interfaces)):
            assert salt.minion.resolve_dns(self.opts) == {
                "master_ip": "127.0.0.1",
                "source_ip": "111.1.0.1",
                "source_ret_port": 49017,
                "source_publish_port": 49018,
                "master_uri": "tcp://127.0.0.1:4555",
            }

    @slowTest
    def test_source_int_name_remote(self):
        """
        test when file_client remote and
        source_interface_name is set and
        interface is down
        """
        interfaces = {
            "bond0.1234": {
                "hwaddr": "01:01:01:d0:d0:d0",
                "up": False,
                "inet": [
                    {
                        "broadcast": "111.1.111.255",
                        "netmask": "111.1.0.0",
                        "label": "bond0",
                        "address": "111.1.0.1",
                    }
                ],
            }
        }
        with patch.dict(
            self.opts,
            {
                "ipv6": False,
                "master": "127.0.0.1",
                "master_port": "4555",
                "file_client": "remote",
                "source_interface_name": "bond0.1234",
                "source_ret_port": 49017,
                "source_publish_port": 49018,
            },
        ), patch("salt.utils.network.interfaces", MagicMock(return_value=interfaces)):
            assert salt.minion.resolve_dns(self.opts) == {
                "master_ip": "127.0.0.1",
                "source_ret_port": 49017,
                "source_publish_port": 49018,
                "master_uri": "tcp://127.0.0.1:4555",
            }

    @slowTest
    def test_source_address(self):
        """
        test when source_address is set
        """
        interfaces = {
            "bond0.1234": {
                "hwaddr": "01:01:01:d0:d0:d0",
                "up": False,
                "inet": [
                    {
                        "broadcast": "111.1.111.255",
                        "netmask": "111.1.0.0",
                        "label": "bond0",
                        "address": "111.1.0.1",
                    }
                ],
            }
        }
        with patch.dict(
            self.opts,
            {
                "ipv6": False,
                "master": "127.0.0.1",
                "master_port": "4555",
                "file_client": "local",
                "source_interface_name": "",
                "source_address": "111.1.0.1",
                "source_ret_port": 49017,
                "source_publish_port": 49018,
            },
        ), patch("salt.utils.network.interfaces", MagicMock(return_value=interfaces)):
            assert salt.minion.resolve_dns(self.opts) == {
                "source_publish_port": 49018,
                "source_ret_port": 49017,
                "master_uri": "tcp://127.0.0.1:4555",
                "source_ip": "111.1.0.1",
                "master_ip": "127.0.0.1",
            }

    # Tests for _handle_decoded_payload in the salt.minion.Minion() class: 3

    @slowTest
    def test_handle_decoded_payload_jid_match_in_jid_queue(self):
        """
        Tests that the _handle_decoded_payload function returns when a jid is given that is already present
        in the jid_queue.

        Note: This test doesn't contain all of the patch decorators above the function like the other tests
        for _handle_decoded_payload below. This is essential to this test as the call to the function must
        return None BEFORE any of the processes are spun up because we should be avoiding firing duplicate
        jobs.
        """
        mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
        mock_data = {"fun": "foo.bar", "jid": 123}
        mock_jid_queue = [123]
        minion = salt.minion.Minion(
            mock_opts,
            jid_queue=copy.copy(mock_jid_queue),
            io_loop=salt.ext.tornado.ioloop.IOLoop(),
        )
        try:
            ret = minion._handle_decoded_payload(mock_data).result()
            self.assertEqual(minion.jid_queue, mock_jid_queue)
            self.assertIsNone(ret)
        finally:
            minion.destroy()

    @slowTest
    def test_handle_decoded_payload_jid_queue_addition(self):
        """
        Tests that the _handle_decoded_payload function adds a jid to the minion's jid_queue when the new
        jid isn't already present in the jid_queue.
        """
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.utils.process.SignalHandlingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.join",
            MagicMock(return_value=True),
        ):
            mock_jid = 11111
            mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
            mock_data = {"fun": "foo.bar", "jid": mock_jid}
            mock_jid_queue = [123, 456]
            minion = salt.minion.Minion(
                mock_opts,
                jid_queue=copy.copy(mock_jid_queue),
                io_loop=salt.ext.tornado.ioloop.IOLoop(),
            )
            try:

                # Assert that the minion's jid_queue attribute matches the mock_jid_queue as a baseline
                # This can help debug any test failures if the _handle_decoded_payload call fails.
                self.assertEqual(minion.jid_queue, mock_jid_queue)

                # Call the _handle_decoded_payload function and update the mock_jid_queue to include the new
                # mock_jid. The mock_jid should have been added to the jid_queue since the mock_jid wasn't
                # previously included. The minion's jid_queue attribute and the mock_jid_queue should be equal.
                minion._handle_decoded_payload(mock_data).result()
                mock_jid_queue.append(mock_jid)
                self.assertEqual(minion.jid_queue, mock_jid_queue)
            finally:
                minion.destroy()

    @slowTest
    def test_handle_decoded_payload_jid_queue_reduced_minion_jid_queue_hwm(self):
        """
        Tests that the _handle_decoded_payload function removes a jid from the minion's jid_queue when the
        minion's jid_queue high water mark (minion_jid_queue_hwm) is hit.
        """
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.utils.process.SignalHandlingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.join",
            MagicMock(return_value=True),
        ):
            mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
            mock_opts["minion_jid_queue_hwm"] = 2
            mock_data = {"fun": "foo.bar", "jid": 789}
            mock_jid_queue = [123, 456]
            minion = salt.minion.Minion(
                mock_opts,
                jid_queue=copy.copy(mock_jid_queue),
                io_loop=salt.ext.tornado.ioloop.IOLoop(),
            )
            try:

                # Assert that the minion's jid_queue attribute matches the mock_jid_queue as a baseline
                # This can help debug any test failures if the _handle_decoded_payload call fails.
                self.assertEqual(minion.jid_queue, mock_jid_queue)

                # Call the _handle_decoded_payload function and check that the queue is smaller by one item
                # and contains the new jid
                minion._handle_decoded_payload(mock_data).result()
                self.assertEqual(len(minion.jid_queue), 2)
                self.assertEqual(minion.jid_queue, [456, 789])
            finally:
                minion.destroy()

    @slowTest
    def test_process_count_max(self):
        """
        Tests that the _handle_decoded_payload function does not spawn more than the configured amount of processes,
        as per process_count_max.
        """
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.utils.process.SignalHandlingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.join",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.minion.running", MagicMock(return_value=[])
        ), patch(
            "salt.ext.tornado.gen.sleep",
            MagicMock(return_value=salt.ext.tornado.concurrent.Future()),
        ):
            process_count_max = 10
            mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
            mock_opts["__role"] = "minion"
            mock_opts["minion_jid_queue_hwm"] = 100
            mock_opts["process_count_max"] = process_count_max

            io_loop = salt.ext.tornado.ioloop.IOLoop()
            minion = salt.minion.Minion(mock_opts, jid_queue=[], io_loop=io_loop)
            try:

                # mock gen.sleep to throw a special Exception when called, so that we detect it
                class SleepCalledException(Exception):
                    """Thrown when sleep is called"""

                salt.ext.tornado.gen.sleep.return_value.set_exception(
                    SleepCalledException()
                )

                # up until process_count_max: gen.sleep does not get called, processes are started normally
                for i in range(process_count_max):
                    mock_data = {"fun": "foo.bar", "jid": i}
                    io_loop.run_sync(
                        lambda data=mock_data: minion._handle_decoded_payload(data)
                    )
                    self.assertEqual(
                        salt.utils.process.SignalHandlingProcess.start.call_count, i + 1
                    )
                    self.assertEqual(len(minion.jid_queue), i + 1)
                    salt.utils.minion.running.return_value += [i]

                # above process_count_max: gen.sleep does get called, JIDs are created but no new processes are started
                mock_data = {"fun": "foo.bar", "jid": process_count_max + 1}

                self.assertRaises(
                    SleepCalledException,
                    lambda: io_loop.run_sync(
                        lambda: minion._handle_decoded_payload(mock_data)
                    ),
                )
                self.assertEqual(
                    salt.utils.process.SignalHandlingProcess.start.call_count,
                    process_count_max,
                )
                self.assertEqual(len(minion.jid_queue), process_count_max + 1)
            finally:
                minion.destroy()

    @slowTest
    def test_beacons_before_connect(self):
        """
        Tests that the 'beacons_before_connect' option causes the beacons to be initialized before connect.
        """
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.minion.Minion.sync_connect_master",
            MagicMock(side_effect=RuntimeError("stop execution")),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.join",
            MagicMock(return_value=True),
        ):
            mock_opts = self.get_config("minion", from_scratch=True)
            mock_opts["beacons_before_connect"] = True
            io_loop = salt.ext.tornado.ioloop.IOLoop()
            io_loop.make_current()
            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
            try:

                try:
                    minion.tune_in(start=True)
                except RuntimeError:
                    pass

                # Make sure beacons are initialized but the sheduler is not
                self.assertTrue("beacons" in minion.periodic_callbacks)
                self.assertTrue("schedule" not in minion.periodic_callbacks)
            finally:
                minion.destroy()

    @slowTest
    def test_scheduler_before_connect(self):
        """
        Tests that the 'scheduler_before_connect' option causes the scheduler to be initialized before connect.
        """
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.minion.Minion.sync_connect_master",
            MagicMock(side_effect=RuntimeError("stop execution")),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.join",
            MagicMock(return_value=True),
        ):
            mock_opts = self.get_config("minion", from_scratch=True)
            mock_opts["scheduler_before_connect"] = True
            io_loop = salt.ext.tornado.ioloop.IOLoop()
            io_loop.make_current()
            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
            try:
                try:
                    minion.tune_in(start=True)
                except RuntimeError:
                    pass

                # Make sure the scheduler is initialized but the beacons are not
                self.assertTrue("schedule" in minion.periodic_callbacks)
                self.assertTrue("beacons" not in minion.periodic_callbacks)
            finally:
                minion.destroy()

    @slowTest
    def test_when_ping_interval_is_set_the_callback_should_be_added_to_periodic_callbacks(
        self,
    ):
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.minion.Minion.sync_connect_master",
            MagicMock(side_effect=RuntimeError("stop execution")),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingProcess.join",
            MagicMock(return_value=True),
        ):
            mock_opts = self.get_config("minion", from_scratch=True)
            mock_opts["ping_interval"] = 10
            io_loop = salt.ext.tornado.ioloop.IOLoop()
            io_loop.make_current()
            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
            try:
                try:
                    minion.connected = MagicMock(side_effect=(False, True))
                    minion._fire_master_minion_start = MagicMock()
                    minion.tune_in(start=False)
                except RuntimeError:
                    pass

                # Make sure the scheduler is initialized but the beacons are not
                self.assertTrue("ping" in minion.periodic_callbacks)
            finally:
                minion.destroy()

    @slowTest
    def test_when_passed_start_event_grains(self):
        mock_opts = self.get_config("minion", from_scratch=True)
        mock_opts["start_event_grains"] = ["os"]
        io_loop = salt.ext.tornado.ioloop.IOLoop()
        io_loop.make_current()
        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
        try:
            minion.tok = MagicMock()
            minion._send_req_sync = MagicMock()
            minion._fire_master(
                "Minion has started", "minion_start", include_startup_grains=True
            )
            load = minion._send_req_sync.call_args[0][0]

            self.assertTrue("grains" in load)
            self.assertTrue("os" in load["grains"])
        finally:
            minion.destroy()

    @slowTest
    def test_when_not_passed_start_event_grains(self):
        mock_opts = self.get_config("minion", from_scratch=True)
        io_loop = salt.ext.tornado.ioloop.IOLoop()
        io_loop.make_current()
        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
        try:
            minion.tok = MagicMock()
            minion._send_req_sync = MagicMock()
            minion._fire_master("Minion has started", "minion_start")
            load = minion._send_req_sync.call_args[0][0]

            self.assertTrue("grains" not in load)
        finally:
            minion.destroy()

    @slowTest
    def test_when_other_events_fired_and_start_event_grains_are_set(self):
        mock_opts = self.get_config("minion", from_scratch=True)
        mock_opts["start_event_grains"] = ["os"]
        io_loop = salt.ext.tornado.ioloop.IOLoop()
        io_loop.make_current()
        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
        try:
            minion.tok = MagicMock()
            minion._send_req_sync = MagicMock()
            minion._fire_master("Custm_event_fired", "custom_event")
            load = minion._send_req_sync.call_args[0][0]

            self.assertTrue("grains" not in load)
        finally:
            minion.destroy()

    @slowTest
    def test_minion_retry_dns_count(self):
        """
        Tests that the resolve_dns will retry dns look ups for a maximum of
        3 times before raising a SaltMasterUnresolvableError exception.
        """
        with patch.dict(
            self.opts,
            {
                "ipv6": False,
                "master": "dummy",
                "master_port": "4555",
                "retry_dns": 1,
                "retry_dns_count": 3,
            },
        ):
            self.assertRaises(
                SaltMasterUnresolvableError, salt.minion.resolve_dns, self.opts
            )

    @slowTest
    def test_gen_modules_executors(self):
        """
        Ensure gen_modules is called with the correct arguments #54429
        """
        mock_opts = self.get_config("minion", from_scratch=True)
        io_loop = salt.ext.tornado.ioloop.IOLoop()
        io_loop.make_current()
        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)

        class MockPillarCompiler(object):
            def compile_pillar(self):
                return {}

        try:
            with patch("salt.pillar.get_pillar", return_value=MockPillarCompiler()):
                with patch("salt.loader.executors") as execmock:
                    minion.gen_modules()
            assert execmock.called_with(minion.opts, minion.functions)
        finally:
            minion.destroy()

    @patch("salt.utils.process.default_signals")
    @slowTest
    def test_reinit_crypto_on_fork(self, def_mock):
        """
        Ensure salt.utils.crypt.reinit_crypto() is executed when forking for new job
        """
        mock_opts = self.get_config("minion", from_scratch=True)
        mock_opts["multiprocessing"] = True

        io_loop = salt.ext.tornado.ioloop.IOLoop()
        io_loop.make_current()
        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)

        job_data = {"jid": "test-jid", "fun": "test.ping"}

        def mock_start(self):
            # pylint: disable=comparison-with-callable
            assert (
                len(
                    [
                        x
                        for x in self._after_fork_methods
                        if x[0] == salt.utils.crypt.reinit_crypto
                    ]
                )
                == 1
            )
            # pylint: enable=comparison-with-callable

        with patch.object(
            salt.utils.process.SignalHandlingProcess, "start", mock_start
        ):
            io_loop.run_sync(lambda: minion._handle_decoded_payload(job_data))

    def test_minion_manage_schedule(self):
        """
        Tests that the manage_schedule will call the add function, adding
        schedule data into opts.
        """
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.minion.Minion.sync_connect_master",
            MagicMock(side_effect=RuntimeError("stop execution")),
        ), patch(
            "salt.utils.process.SignalHandlingMultiprocessingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingMultiprocessingProcess.join",
            MagicMock(return_value=True),
        ):
            mock_opts = self.get_config("minion", from_scratch=True)
            io_loop = salt.ext.tornado.ioloop.IOLoop()
            io_loop.make_current()

            with patch(
                "salt.utils.schedule.clean_proc_dir", MagicMock(return_value=None)
            ):
                mock_functions = {"test.ping": None}

                minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
                minion.schedule = salt.utils.schedule.Schedule(
                    mock_opts, mock_functions, returners={}
                )

                schedule_data = {
                    "test_job": {
                        "function": "test.ping",
                        "return_job": False,
                        "jid_include": True,
                        "maxrunning": 2,
                        "seconds": 10,
                    }
                }

                data = {"name": "test-item", "schedule": schedule_data, "func": "add"}
                tag = "manage_schedule"

                minion.manage_schedule(tag, data)
                self.assertIn("test_job", minion.opts["schedule"])

    def test_minion_manage_beacons(self):
        """
        Tests that the manage_beacons will call the add function, adding
        beacon data into opts.
        """
        with patch("salt.minion.Minion.ctx", MagicMock(return_value={})), patch(
            "salt.minion.Minion.sync_connect_master",
            MagicMock(side_effect=RuntimeError("stop execution")),
        ), patch(
            "salt.utils.process.SignalHandlingMultiprocessingProcess.start",
            MagicMock(return_value=True),
        ), patch(
            "salt.utils.process.SignalHandlingMultiprocessingProcess.join",
            MagicMock(return_value=True),
        ):
            mock_opts = self.get_config("minion", from_scratch=True)
            io_loop = salt.ext.tornado.ioloop.IOLoop()
            io_loop.make_current()

            mock_functions = {"test.ping": None}
            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
            minion.beacons = salt.beacons.Beacon(mock_opts, mock_functions)

            bdata = [{"salt-master": "stopped"}, {"apache2": "stopped"}]
            data = {"name": "ps", "beacon_data": bdata, "func": "add"}

            tag = "manage_beacons"

            minion.manage_beacons(tag, data)
            self.assertIn("ps", minion.opts["beacons"])
            self.assertEqual(minion.opts["beacons"]["ps"], bdata)


class MinionAsyncTestCase(
    TestCase, AdaptedConfigurationTestCaseMixin, salt.ext.tornado.testing.AsyncTestCase
):
    def setUp(self):
        super(MinionAsyncTestCase, self).setUp()
        self.opts = {}
        self.addCleanup(delattr, self, "opts")

    @skip_if_not_root
    def test_sock_path_len(self):
        """
        This tests whether or not a larger hash causes the sock path to exceed
        the system's max sock path length. See the below link for more
        information.

        https://github.com/saltstack/salt/issues/12172#issuecomment-43903643
        """
        opts = {
            "id": "salt-testing",
            "hash_type": "sha512",
            "sock_dir": os.path.join(salt.syspaths.SOCK_DIR, "minion"),
            "extension_modules": "",
        }
        with patch.dict(self.opts, opts):
            try:
                event_publisher = event.AsyncEventPublisher(self.opts)
                result = True
            except ValueError:
                #  There are rare cases where we operate a closed socket, especially in containers.
                # In this case, don't fail the test because we'll catch it down the road.
                result = True
            except SaltSystemExit:
                result = False
        self.assertTrue(result)



# -*- coding: utf-8 -*-
"""
salt-ssh testing
"""
from __future__ import absolute_import, print_function, unicode_literals

import os
import shutil

from tests.support.case import SSHCase
from tests.support.helpers import slowTest
from tests.support.runtests import RUNTIME_VARS


class SSHTest(SSHCase):
    """
    Test general salt-ssh functionality
    """

    @slowTest
    def test_ping(self):
        """
        Test a simple ping
        """
        ret = self.run_function("test.ping")
        self.assertTrue(ret, "Ping did not return true")

    @slowTest
    def test_thin_dir(self):
        """
        test to make sure thin_dir is created
        and salt-call file is included
        """
        thin_dir = self.run_function("config.get", ["thin_dir"], wipe=False)
        os.path.isdir(thin_dir)
        os.path.exists(os.path.join(thin_dir, "salt-call"))
        os.path.exists(os.path.join(thin_dir, "running_data"))

    def test_set_path(self):
        """
        test setting the path env variable
        """
        path = "/pathdoesnotexist/"
        roster = os.path.join(RUNTIME_VARS.TMP, "roster-set-path")
        self.custom_roster(
            roster, data={"set_path": "$PATH:/usr/local/bin/:{0}".format(path)}
        )
        ret = self.run_function("environ.get", ["PATH"], roster_file=roster)
        assert path in ret

    def tearDown(self):
        """
        make sure to clean up any old ssh directories
        """
        salt_dir = self.run_function("config.get", ["thin_dir"], wipe=False)
        if os.path.exists(salt_dir):
            shutil.rmtree(salt_dir)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.eselect as eselect

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class EselectTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.eselect
    """

    def setup_loader_modules(self):
        return {eselect: {}}

    # 'set_' function tests: 1

    def test_set_(self):
        """
        Test to verify that the given module is set to the given target
        """
        name = "myeselect"
        target = "hardened/linux/amd64"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(return_value=target)
        with patch.dict(eselect.__salt__, {"eselect.get_current_target": mock}):
            comt = "Target '{0}' is already set on '{1}' module.".format(target, name)
            ret.update({"comment": comt})
            self.assertDictEqual(eselect.set_(name, target), ret)



# -*- coding: utf-8 -*-
"""
Management of MySQL databases (schemas)
=======================================

:depends:   - MySQLdb Python module
:configuration: See :py:mod:`salt.modules.mysql` for setup instructions.

The mysql_database module is used to create and manage MySQL databases.
Databases can be set as either absent or present.

.. code-block:: yaml

    frank:
      mysql_database.present
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging
import sys

log = logging.getLogger(__name__)


def __virtual__():
    """
    Only load if the mysql module is available in __salt__
    """
    if "mysql.db_exists" in __salt__:
        return True
    return (False, "mysql module could not be loaded")


def _get_mysql_error():
    """
    Look in module context for a MySQL error. Eventually we should make a less
    ugly way of doing this.
    """
    return sys.modules[__salt__["test.ping"].__module__].__context__.pop(
        "mysql.error", None
    )


def present(name, character_set=None, collate=None, **connection_args):
    """
    Ensure that the named database is present with the specified properties

    name
        The name of the database to manage
    """
    ret = {
        "name": name,
        "changes": {},
        "result": True,
        "comment": "Database {0} is already present".format(name),
    }
    # check if database exists
    existing = __salt__["mysql.db_get"](name, **connection_args)
    if existing:
        alter = False
        if character_set and character_set != existing.get("character_set"):
            log.debug(
                "character set differes from %s : %s",
                character_set,
                existing.get("character_set"),
            )
            alter = True
        if collate and collate != existing.get("collate"):
            log.debug(
                "collate set differs from %s : %s", collate, existing.get("collate")
            )
            alter = True
        if alter:
            __salt__["mysql.alter_db"](
                name, character_set=character_set, collate=collate, **connection_args
            )
        current = __salt__["mysql.db_get"](name, **connection_args)
        if existing.get("collate", None) != current.get("collate", None):
            ret["changes"].update(
                {
                    "collate": {
                        "before": existing.get("collate", None),
                        "now": current.get("collate", None),
                    }
                }
            )
        if existing.get("character_set", None) != current.get("character_set", None):
            ret["changes"].update(
                {
                    "character_set": {
                        "before": existing.get("character_set", None),
                        "now": current.get("character_set", None),
                    }
                }
            )
        return ret
    else:
        err = _get_mysql_error()
        if err is not None:
            ret["comment"] = err
            ret["result"] = False
            return ret

    if __opts__["test"]:
        ret["result"] = None
        ret["comment"] = ("Database {0} is not present and needs to be created").format(
            name
        )
        return ret
    # The database is not present, make it!
    if __salt__["mysql.db_create"](
        name, character_set=character_set, collate=collate, **connection_args
    ):
        ret["comment"] = "The database {0} has been created".format(name)
        ret["changes"][name] = "Present"
    else:
        ret["comment"] = "Failed to create database {0}".format(name)
        err = _get_mysql_error()
        if err is not None:
            ret["comment"] += " ({0})".format(err)
        ret["result"] = False

    return ret


def absent(name, **connection_args):
    """
    Ensure that the named database is absent

    name
        The name of the database to remove
    """
    ret = {"name": name, "changes": {}, "result": True, "comment": ""}

    # check if db exists and remove it
    if __salt__["mysql.db_exists"](name, **connection_args):
        if __opts__["test"]:
            ret["result"] = None
            ret["comment"] = "Database {0} is present and needs to be removed".format(
                name
            )
            return ret
        if __salt__["mysql.db_remove"](name, **connection_args):
            ret["comment"] = "Database {0} has been removed".format(name)
            ret["changes"][name] = "Absent"
            return ret
        else:
            err = _get_mysql_error()
            if err is not None:
                ret["comment"] = "Unable to remove database {0} " "({1})".format(
                    name, err
                )
                ret["result"] = False
                return ret
    else:
        err = _get_mysql_error()
        if err is not None:
            ret["comment"] = err
            ret["result"] = False
            return ret

    # fallback
    ret["comment"] = ("Database {0} is not present, so it cannot be removed").format(
        name
    )
    return ret



# -*- coding: utf-8 -*-
"""
Common resources for LXC and systemd-nspawn containers

.. versionadded:: 2015.8.0

These functions are not designed to be called directly, but instead from the
:mod:`lxc <salt.modules.lxc>`, :mod:`nspawn <salt.modules.nspawn>`, and
:mod:`docker <salt.modules.docker>` execution modules. They provide for
common logic to be re-used for common actions.
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import copy
import functools
import logging
import os
import pipes
import time
import traceback

# Import salt libs
import salt.utils.args
import salt.utils.path
import salt.utils.vt
from salt.exceptions import CommandExecutionError, SaltInvocationError

log = logging.getLogger(__name__)

PATH = "PATH=/bin:/usr/bin:/sbin:/usr/sbin:/opt/bin:" "/usr/local/bin:/usr/local/sbin"


def _validate(wrapped):
    """
    Decorator for common function argument validation
    """

    @functools.wraps(wrapped)
    def wrapper(*args, **kwargs):
        container_type = kwargs.get("container_type")
        exec_driver = kwargs.get("exec_driver")
        valid_driver = {
            "docker": ("lxc-attach", "nsenter", "docker-exec"),
            "lxc": ("lxc-attach",),
            "nspawn": ("nsenter",),
        }
        if container_type not in valid_driver:
            raise SaltInvocationError(
                "Invalid container type '{0}'. Valid types are: {1}".format(
                    container_type, ", ".join(sorted(valid_driver))
                )
            )
        if exec_driver not in valid_driver[container_type]:
            raise SaltInvocationError(
                "Invalid command execution driver. Valid drivers are: {0}".format(
                    ", ".join(valid_driver[container_type])
                )
            )
        if exec_driver == "lxc-attach" and not salt.utils.path.which("lxc-attach"):
            raise SaltInvocationError(
                "The 'lxc-attach' execution driver has been chosen, but "
                "lxc-attach is not available. LXC may not be installed."
            )
        return wrapped(*args, **salt.utils.args.clean_kwargs(**kwargs))

    return wrapper


def _nsenter(pid):
    """
    Return the nsenter command to attach to the named container
    """
    return "nsenter --target {0} --mount --uts --ipc --net --pid".format(pid)


def _get_md5(name, path, run_func):
    """
    Get the MD5 checksum of a file from a container
    """
    output = run_func(
        name, "md5sum {0}".format(pipes.quote(path)), ignore_retcode=True
    )["stdout"]
    try:
        return output.split()[0]
    except IndexError:
        # Destination file does not exist or could not be accessed
        return None


def cache_file(source):
    """
    Wrapper for cp.cache_file which raises an error if the file was unable to
    be cached.

    CLI Example:

    .. code-block:: bash

        salt myminion container_resource.cache_file salt://foo/bar/baz.txt
    """
    try:
        # Don't just use cp.cache_file for this. Docker has its own code to
        # pull down images from the web.
        if source.startswith("salt://"):
            cached_source = __salt__["cp.cache_file"](source)
            if not cached_source:
                raise CommandExecutionError("Unable to cache {0}".format(source))
            return cached_source
    except AttributeError:
        raise SaltInvocationError("Invalid source file {0}".format(source))
    return source


@_validate
def run(
    name,
    cmd,
    container_type=None,
    exec_driver=None,
    output=None,
    no_start=False,
    stdin=None,
    python_shell=True,
    output_loglevel="debug",
    ignore_retcode=False,
    path=None,
    use_vt=False,
    keep_env=None,
):
    """
    Common logic for running shell commands in containers

    path
        path to the container parent (for LXC only)
        default: /var/lib/lxc (system default)

    CLI Example:

    .. code-block:: bash

        salt myminion container_resource.run mycontainer 'ps aux' container_type=docker exec_driver=nsenter output=stdout
    """
    valid_output = ("stdout", "stderr", "retcode", "all")
    if output is None:
        cmd_func = "cmd.run"
    elif output not in valid_output:
        raise SaltInvocationError(
            "'output' param must be one of the following: {0}".format(
                ", ".join(valid_output)
            )
        )
    else:
        cmd_func = "cmd.run_all"

    if keep_env is None or isinstance(keep_env, bool):
        to_keep = []
    elif not isinstance(keep_env, (list, tuple)):
        try:
            to_keep = keep_env.split(",")
        except AttributeError:
            log.warning("Invalid keep_env value, ignoring")
            to_keep = []
    else:
        to_keep = keep_env

    if exec_driver == "lxc-attach":
        full_cmd = "lxc-attach "
        if path:
            full_cmd += "-P {0} ".format(pipes.quote(path))
        if keep_env is not True:
            full_cmd += "--clear-env "
            if "PATH" not in to_keep:
                full_cmd += "--set-var {0} ".format(PATH)
                # --clear-env results in a very restrictive PATH
                # (/bin:/usr/bin), use a good fallback.
        full_cmd += " ".join(
            [
                "--set-var {0}={1}".format(x, pipes.quote(os.environ[x]))
                for x in to_keep
                if x in os.environ
            ]
        )
        full_cmd += " -n {0} -- {1}".format(pipes.quote(name), cmd)
    elif exec_driver == "nsenter":
        pid = __salt__["{0}.pid".format(container_type)](name)
        full_cmd = "nsenter --target {0} --mount --uts --ipc --net --pid -- ".format(
            pid
        )
        if keep_env is not True:
            full_cmd += "env -i "
            if "PATH" not in to_keep:
                full_cmd += "{0} ".format(PATH)
        full_cmd += " ".join(
            [
                "{0}={1}".format(x, pipes.quote(os.environ[x]))
                for x in to_keep
                if x in os.environ
            ]
        )
        full_cmd += " {0}".format(cmd)
    elif exec_driver == "docker-exec":
        # We're using docker exec on the CLI as opposed to via docker-py, since
        # the Docker API doesn't return stdout and stderr separately.
        full_cmd = "docker exec "
        if stdin:
            full_cmd += "-i "
        full_cmd += "{0} ".format(name)
        if keep_env is not True:
            full_cmd += "env -i "
            if "PATH" not in to_keep:
                full_cmd += "{0} ".format(PATH)
        full_cmd += " ".join(
            [
                "{0}={1}".format(x, pipes.quote(os.environ[x]))
                for x in to_keep
                if x in os.environ
            ]
        )
        full_cmd += " {0}".format(cmd)

    if not use_vt:
        ret = __salt__[cmd_func](
            full_cmd,
            stdin=stdin,
            python_shell=python_shell,
            output_loglevel=output_loglevel,
            ignore_retcode=ignore_retcode,
        )
    else:
        stdout, stderr = "", ""
        proc = salt.utils.vt.Terminal(
            full_cmd,
            shell=python_shell,
            log_stdin_level="quiet" if output_loglevel == "quiet" else "info",
            log_stdout_level=output_loglevel,
            log_stderr_level=output_loglevel,
            log_stdout=True,
            log_stderr=True,
            stream_stdout=False,
            stream_stderr=False,
        )
        # Consume output
        try:
            while proc.has_unread_data:
                try:
                    cstdout, cstderr = proc.recv()
                    if cstdout:
                        stdout += cstdout
                    if cstderr:
                        if output is None:
                            stdout += cstderr
                        else:
                            stderr += cstderr
                    time.sleep(0.5)
                except KeyboardInterrupt:
                    break
            ret = (
                stdout
                if output is None
                else {
                    "retcode": proc.exitstatus,
                    "pid": 2,
                    "stdout": stdout,
                    "stderr": stderr,
                }
            )
        except salt.utils.vt.TerminalException:
            trace = traceback.format_exc()
            log.error(trace)
            ret = (
                stdout
                if output is None
                else {"retcode": 127, "pid": 2, "stdout": stdout, "stderr": stderr}
            )
        finally:
            proc.terminate()

    return ret


@_validate
def copy_to(
    name,
    source,
    dest,
    container_type=None,
    path=None,
    exec_driver=None,
    overwrite=False,
    makedirs=False,
):
    """
    Common logic for copying files to containers

    path
        path to the container parent (for LXC only)
        default: /var/lib/lxc (system default)

    CLI Example:

    .. code-block:: bash

        salt myminion container_resource.copy_to mycontainer /local/file/path /container/file/path container_type=docker exec_driver=nsenter
    """
    # Get the appropriate functions
    state = __salt__["{0}.state".format(container_type)]

    def run_all(*args, **akwargs):
        akwargs = copy.deepcopy(akwargs)
        if container_type in ["lxc"] and "path" not in akwargs:
            akwargs["path"] = path
        return __salt__["{0}.run_all".format(container_type)](*args, **akwargs)

    state_kwargs = {}
    cmd_kwargs = {"ignore_retcode": True}
    if container_type in ["lxc"]:
        cmd_kwargs["path"] = path
        state_kwargs["path"] = path

    def _state(name):
        if state_kwargs:
            return state(name, **state_kwargs)
        else:
            return state(name)

    c_state = _state(name)
    if c_state != "running":
        raise CommandExecutionError("Container '{0}' is not running".format(name))

    local_file = cache_file(source)
    source_dir, source_name = os.path.split(local_file)

    # Source file sanity checks
    if not os.path.isabs(local_file):
        raise SaltInvocationError("Source path must be absolute")
    elif not os.path.exists(local_file):
        raise SaltInvocationError("Source file {0} does not exist".format(local_file))
    elif not os.path.isfile(local_file):
        raise SaltInvocationError("Source must be a regular file")

    # Destination file sanity checks
    if not os.path.isabs(dest):
        raise SaltInvocationError("Destination path must be absolute")
    if (
        run_all(name, "test -d {0}".format(pipes.quote(dest)), **cmd_kwargs)["retcode"]
        == 0
    ):
        # Destination is a directory, full path to dest file will include the
        # basename of the source file.
        dest = os.path.join(dest, source_name)
    else:
        # Destination was not a directory. We will check to see if the parent
        # dir is a directory, and then (if makedirs=True) attempt to create the
        # parent directory.
        dest_dir, dest_name = os.path.split(dest)
        if (
            run_all(name, "test -d {0}".format(pipes.quote(dest_dir)), **cmd_kwargs)[
                "retcode"
            ]
            != 0
        ):
            if makedirs:
                result = run_all(
                    name, "mkdir -p {0}".format(pipes.quote(dest_dir)), **cmd_kwargs
                )
                if result["retcode"] != 0:
                    error = (
                        "Unable to create destination directory {0} in "
                        "container '{1}'".format(dest_dir, name)
                    )
                    if result["stderr"]:
                        error += ": {0}".format(result["stderr"])
                    raise CommandExecutionError(error)
            else:
                raise SaltInvocationError(
                    "Directory {0} does not exist on {1} container '{2}'".format(
                        dest_dir, container_type, name
                    )
                )
    if (
        not overwrite
        and run_all(name, "test -e {0}".format(pipes.quote(dest)), **cmd_kwargs)[
            "retcode"
        ]
        == 0
    ):
        raise CommandExecutionError(
            "Destination path {0} already exists. Use overwrite=True to "
            "overwrite it".format(dest)
        )

    # Before we try to replace the file, compare checksums.
    source_md5 = __salt__["file.get_sum"](local_file, "md5")
    if source_md5 == _get_md5(name, dest, run_all):
        log.debug("%s and %s:%s are the same file, skipping copy", source, name, dest)
        return True

    log.debug(
        "Copying %s to %s container '%s' as %s", source, container_type, name, dest
    )

    # Using cat here instead of opening the file, reading it into memory,
    # and passing it as stdin to run(). This will keep down memory
    # usage for the minion and make the operation run quicker.
    if exec_driver == "lxc-attach":
        lxcattach = "lxc-attach"
        if path:
            lxcattach += " -P {0}".format(pipes.quote(path))
        copy_cmd = (
            'cat "{0}" | {4} --clear-env --set-var {1} -n {2} -- '
            'tee "{3}"'.format(local_file, PATH, name, dest, lxcattach)
        )
    elif exec_driver == "nsenter":
        pid = __salt__["{0}.pid".format(container_type)](name)
        copy_cmd = 'cat "{0}" | {1} env -i {2} tee "{3}"'.format(
            local_file, _nsenter(pid), PATH, dest
        )
    elif exec_driver == "docker-exec":
        copy_cmd = 'cat "{0}" | docker exec -i {1} env -i {2} tee "{3}"'.format(
            local_file, name, PATH, dest
        )
    __salt__["cmd.run"](copy_cmd, python_shell=True, output_loglevel="quiet")
    return source_md5 == _get_md5(name, dest, run_all)



# -*- coding: utf-8 -*-
"""
:codeauthor: Herbert Buurman <herbert.buurman@ogd.nl>
"""

# Import future libs
from __future__ import absolute_import, print_function, unicode_literals

import datetime
import os

# Import Python libs
import textwrap

# Import Salt Module
import salt.modules.acme as acme
import salt.utils.dictupdate
from salt.exceptions import SaltInvocationError

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AcmeTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.acme
    """

    def setup_loader_modules(self):
        return {acme: {}}

    def test_certs(self):
        """
        Test listing certs
        """
        with patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "file.readdir": MagicMock(
                    return_value=[".", "..", "README", "test_expired", "test_valid"]
                )
            },
        ), patch(
            "os.path.isdir",
            side_effect=lambda path: path
            in [
                os.path.join(acme.LE_LIVE, "test_expired"),
                os.path.join(acme.LE_LIVE, "test_valid"),
            ],
        ):
            self.assertEqual(acme.certs(), ["test_expired", "test_valid"])

    def test_has(self):
        """
        Test checking if certificate (does not) exist.
        """
        with patch.dict(
            acme.__salt__, {"file.file_exists": MagicMock(return_value=True)}
        ):  # pylint: disable=no-member
            self.assertTrue(acme.has("test_expired"))
        with patch.dict(
            acme.__salt__, {"file.file_exists": MagicMock(return_value=False)}
        ):  # pylint: disable=no-member
            self.assertFalse(acme.has("test_invalid"))

    def test_needs_renewal(self):
        """
        Test if expired certs do indeed need renewal.
        """
        expired = (
            datetime.date.today()
            - datetime.timedelta(days=3)
            - datetime.date(1970, 1, 1)
        )
        valid = (
            datetime.date.today()
            + datetime.timedelta(days=3)
            - datetime.date(1970, 1, 1)
        )
        with patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "tls.cert_info": MagicMock(
                    return_value={"not_after": expired.total_seconds()}
                )
            },
        ):
            self.assertTrue(acme.needs_renewal("test_expired"))
        with patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "tls.cert_info": MagicMock(
                    return_value={"not_after": valid.total_seconds()}
                )
            },
        ):
            self.assertFalse(acme.needs_renewal("test_valid"))
            # Test with integer window parameter
            self.assertTrue(acme.needs_renewal("test_valid", window=5))
            # Test with string-like window parameter
            self.assertTrue(acme.needs_renewal("test_valid", window="5"))
            # Test with 'force' parameter
            self.assertTrue(acme.needs_renewal("test_valid", window="force"))
            # Test with 'true' parameter
            self.assertTrue(acme.needs_renewal("test_valid", window=True))
            # Test with invalid window parameter
            self.assertRaises(
                SaltInvocationError, acme.needs_renewal, "test_valid", window="foo"
            )

    def test_expires(self):
        """
        Test if expires function functions properly.
        """
        test_value = datetime.datetime.today() - datetime.timedelta(days=3)
        test_stamp = test_value - datetime.datetime(1970, 1, 1)
        with patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "tls.cert_info": MagicMock(
                    return_value={"not_after": test_stamp.total_seconds()}
                )
            },
        ):
            self.assertEqual(
                acme.expires("test_expired"),
                datetime.datetime.fromtimestamp(test_stamp.total_seconds()).isoformat(),
            )

    def test_info(self):
        """
        Test certificate information retrieval.
        """
        certinfo_tls_result = {
            "not_after": 1559471377,
            "signature_algorithm": "sha256WithRSAEncryption",
            "extensions": {},
            "fingerprint": (
                "FB:A4:5F:71:D6:5D:6C:B6:1D:2C:FD:91:09:2C:1C:52:"
                "3C:EC:B6:4D:1A:95:65:37:04:D0:E2:5E:C7:64:0C:9C"
            ),
            "serial_number": 6461481982668892235,
            "issuer": {},
            "not_before": 1559557777,
            "subject": {},
        }
        certinfo_x509_result = {
            "Not After": "2019-06-02 10:29:37",
            "Subject Hash": "54:3B:6C:A4",
            "Serial Number": "59:AB:CB:A0:FB:90:E8:4B",
            "SHA1 Finger Print": "F1:8D:F3:26:1B:D3:88:32:CD:B6:FA:3B:85:58:DA:C7:6F:62:BE:7E",
            "SHA-256 Finger Print": (
                "FB:A4:5F:71:D6:5D:6C:B6:1D:2C:FD:91:09:2C:1C:52:"
                "3C:EC:B6:4D:1A:95:65:37:04:D0:E2:5E:C7:64:0C:9C"
            ),
            "MD5 Finger Print": "95:B5:96:9B:42:A5:9E:20:78:FD:99:09:4B:21:1E:97",
            "Version": 3,
            "Key Size": 2048,
            "Public Key": (
                "-----BEGIN PUBLIC KEY-----\n"
                "MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAsVO2vwQPKU92PSBnuGid\n"
                "k8t6KWVE2jEBM10u7CgqQmD/JCnYflEHAo1nOsD7wxdhBrxhf5Qs+pEX1HOsh8VA\n"
                "HDTim0iE8nQVJ0Iuen2SrwaWMhwKmZTSJRYMgd46oCMi2RdlCvcgF2Hw6RTwF7FT\n"
                "hnksc4HBT91XddnP32N558tOT3YejafQNvClz5WcR+E0JzqGrV/+wfe3o+j/q5eK\n"
                "UowttWazeSMvuROtqj/fEk0rop4D14pgzZqWi30tjwhJNl6fSPFWBrLEHGNyDJ+O\n"
                "zfov0B2MRLJibH7GMkOCwsP2g1lVOReqcml+ju6zAKW8nHBTRg0iXB18Ifxef57Y\n"
                "AQIDAQAB\n"
                "-----END PUBLIC KEY-----\n"
            ),
            "Issuer": {},
            "Issuer Hash": "54:3B:6C:A4",
            "Not Before": "2019-06-03 10:29:37",
            "Subject": {},
        }

        with patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "file.file_exists": MagicMock(return_value=True),
                "tls.cert_info": MagicMock(return_value=certinfo_tls_result),
            },
        ):
            self.assertEqual(acme.info("test"), certinfo_tls_result)
        with patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "file.file_exists": MagicMock(return_value=True),
                "x509.read_certificate": MagicMock(return_value=certinfo_x509_result),
            },
        ):
            self.assertEqual(acme.info("test"), certinfo_x509_result)
        with patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "file.file_exists": MagicMock(return_value=True),
                "cmd.run": MagicMock(return_value="foo"),
            },
        ):
            self.assertEqual(acme.info("test"), {"text": "foo"})

    def test_cert(self):
        """
        Test certificate retrieval/renewal
        """
        valid_timestamp = (
            datetime.datetime.now()
            + datetime.timedelta(days=30)
            - datetime.datetime(1970, 1, 1, 0, 0, 0, 0)
        ).total_seconds()
        expired_timestamp = (
            datetime.datetime.now()
            - datetime.timedelta(days=3)
            - datetime.datetime(1970, 1, 1, 0, 0, 0, 0)
        ).total_seconds()
        cmd_new_cert = {
            "stdout": textwrap.dedent(
                """
                IMPORTANT NOTES:
                 - Congratulations! Your certificate and chain have been saved at:
                   /etc/letsencrypt/live/test/fullchain.pem
                   Your key file has been saved at:
                   /etc/letsencrypt/live/test/privkey.pem
                   Your cert will expire on 2019-08-07. To obtain a new or tweaked
                   version of this certificate in the future, simply run certbot
                   again. To non-interactively renew *all* of your certificates, run
                   "certbot renew"
                 - If you like Certbot, please consider supporting our work by:

                   Donating to ISRG / Let's Encrypt:   https://letsencrypt.org/donate
                   Donating to EFF:                    https://eff.org/donate-le
                """
            ),
            "stderr": textwrap.dedent(
                """
                Saving debug log to /var/log/letsencrypt/letsencrypt.log
                Plugins selected: Authenticator standalone, Installer None
                Starting new HTTPS connection (1): acme-v02.api.letsencrypt.org
                Obtaining a new certificate
                Resetting dropped connection: acme-v02.api.letsencrypt.org
                """
            ),
            "retcode": 0,
        }
        result_new_cert = {
            "comment": "Certificate test obtained",
            "not_after": datetime.datetime.fromtimestamp(valid_timestamp).isoformat(),
            "changes": {"mode": "0640"},
            "result": True,
        }

        cmd_no_renew = {
            "stdout": textwrap.dedent(
                """
                - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                Certificate not yet due for renewal; no action taken.
                - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                """
            ),
            "stderr": textwrap.dedent(
                """Saving debug log to /var/log/letsencrypt/letsencrypt.log
                Plugins selected: Authenticator standalone, Installer None
                Starting new HTTPS connection (1): acme-v02.api.letsencrypt.org
                Cert not yet due for renewal
                Keeping the existing certificate
                """
            ),
            "retcode": 0,
        }
        result_no_renew = {
            "comment": "Certificate "
            + os.path.join("/etc/letsencrypt/live/test", "cert.pem")
            + " unchanged",
            "not_after": datetime.datetime.fromtimestamp(valid_timestamp).isoformat(),
            "changes": {},
            "result": True,
        }
        result_renew = {
            "comment": "Certificate test renewed",
            "not_after": datetime.datetime.fromtimestamp(expired_timestamp).isoformat(),
            "changes": {},
            "result": True,
        }

        # Test fetching new certificate
        with patch("salt.modules.acme.LEA", "certbot"), patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "cmd.run_all": MagicMock(return_value=cmd_new_cert),
                "file.file_exists": MagicMock(return_value=False),
                "tls.cert_info": MagicMock(return_value={"not_after": valid_timestamp}),
                "file.check_perms": MagicMock(
                    side_effect=lambda a, x, b, c, d, follow_symlinks: (
                        salt.utils.dictupdate.set_dict_key_value(
                            x, "changes:mode", "0640"
                        ),
                        None,
                    )
                ),
            },
        ):
            self.assertEqual(acme.cert("test"), result_new_cert)
        # Test not renewing a valid certificate
        with patch("salt.modules.acme.LEA", "certbot"), patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "cmd.run_all": MagicMock(return_value=cmd_no_renew),
                "file.file_exists": MagicMock(return_value=True),
                "tls.cert_info": MagicMock(return_value={"not_after": valid_timestamp}),
                "file.check_perms": MagicMock(
                    side_effect=lambda a, x, b, c, d, follow_symlinks: (
                        salt.utils.dictupdate.set_dict_key_value(x, "result", True),
                        None,
                    )
                ),
            },
        ):
            self.assertEqual(acme.cert("test"), result_no_renew)
        # Test renewing an expired certificate
        with patch("salt.modules.acme.LEA", "certbot"), patch.dict(
            acme.__salt__,
            {  # pylint: disable=no-member
                "cmd.run_all": MagicMock(return_value=cmd_new_cert),
                "file.file_exists": MagicMock(return_value=True),
                "tls.cert_info": MagicMock(
                    return_value={"not_after": expired_timestamp}
                ),
                "file.check_perms": MagicMock(
                    side_effect=lambda a, x, b, c, d, follow_symlinks: (
                        salt.utils.dictupdate.set_dict_key_value(x, "result", True),
                        None,
                    )
                ),
            },
        ):
            self.assertEqual(acme.cert("test"), result_renew)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.postgres_group as postgres_group

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PostgresGroupTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.postgres_group
    """

    def setup_loader_modules(self):
        return {postgres_group: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the named group is present
        with the specified privileges.
        """
        name = "frank"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_t = MagicMock(return_value=True)
        mock = MagicMock(return_value=None)
        with patch.dict(
            postgres_group.__salt__,
            {"postgres.role_get": mock, "postgres.group_create": mock_t},
        ):
            with patch.dict(postgres_group.__opts__, {"test": True}):
                comt = "Group {0} is set to be created".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(postgres_group.present(name), ret)

            with patch.dict(postgres_group.__opts__, {"test": False}):
                comt = "The group {0} has been created".format(name)
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(postgres_group.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that the named group is absent.
        """
        name = "frank"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_t = MagicMock(return_value=True)
        mock = MagicMock(side_effect=[True, True, False])
        with patch.dict(
            postgres_group.__salt__,
            {"postgres.user_exists": mock, "postgres.group_remove": mock_t},
        ):
            with patch.dict(postgres_group.__opts__, {"test": True}):
                comt = "Group {0} is set to be removed".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(postgres_group.absent(name), ret)

            with patch.dict(postgres_group.__opts__, {"test": False}):
                comt = "Group {0} has been removed".format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {name: "Absent"}}
                )
                self.assertDictEqual(postgres_group.absent(name), ret)

            comt = "Group {0} is not present, so it cannot be removed".format(name)
            ret.update({"comment": comt, "result": True, "changes": {}})
            self.assertDictEqual(postgres_group.absent(name), ret)



# -*- coding: utf-8 -*-
import types

from salt.cloud.clouds import azurearm as azure
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, create_autospec
from tests.support.unit import TestCase, skipIf


def copy_func(func, globals=None):
    # I do not know that this is complete, but it's sufficient for now.
    # The key to "moving" the function to another module (or stubbed module)
    # is to update __globals__.

    copied_func = types.FunctionType(
        func.__code__, globals, func.__name__, func.__defaults__, func.__closure__
    )
    copied_func.__module__ = func.__module__
    copied_func.__doc__ = func.__doc__
    copied_func.__kwdefaults__ = func.__kwdefaults__
    copied_func.__dict__.update(func.__dict__)
    return copied_func


def mock_module(mod, sut=None):
    if sut is None:
        sut = [None]

    mock = create_autospec(mod)

    # we need to provide a '__globals__' so functions being tested behave correctly.
    mock_globals = {}

    # exclude the system under test
    for name in sut:
        attr = getattr(mod, name)
        if isinstance(attr, types.FunctionType):
            attr = copy_func(attr, mock_globals)
        setattr(mock, name, attr)

    # fully populate our mock_globals
    for name in mod.__dict__:
        if name in mock.__dict__:
            mock_globals[name] = mock.__dict__[name]
        elif type(getattr(mod, name)) is type(types):  # is a module
            mock_globals[name] = getattr(mock, name)
        else:
            mock_globals[name] = mod.__dict__[name]

    return mock


class AzureTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {azure: {"__opts__": {}, "__active_provider_name__": None}}

    @skipIf(not azure.HAS_LIBS, "azure not available")
    def test_function_signatures(self):
        mock_azure = mock_module(
            azure, sut=["request_instance", "six", "__opts__", "__utils__"]
        )
        mock_azure.create_network_interface.return_value = [
            MagicMock(),
            MagicMock(),
            MagicMock(),
        ]
        mock_azure.salt.utils.stringutils.to_str.return_value = "P4ssw0rd"
        mock_azure.salt.utils.cloud.gen_keys.return_value = [MagicMock(), MagicMock()]
        mock_azure.__opts__["pki_dir"] = None

        mock_azure.request_instance.__globals__[
            "__builtins__"
        ] = mock_azure.request_instance.__globals__["__builtins__"].copy()
        mock_azure.request_instance.__globals__["__builtins__"]["getattr"] = MagicMock()

        mock_azure.__utils__[
            "cloud.fire_event"
        ] = mock_azure.salt.utils.cloud.fire_event
        mock_azure.__utils__[
            "cloud.filter_event"
        ] = mock_azure.salt.utils.cloud.filter_event
        mock_azure.__opts__["sock_dir"] = MagicMock()
        mock_azure.__opts__["transport"] = MagicMock()

        mock_azure.request_instance(
            {"image": "http://img", "storage_account": "blah", "size": ""}
        )

        # we literally only check that a final creation call occurred.
        mock_azure.get_conn.return_value.virtual_machines.create_or_update.assert_called_once()



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.states.svn as svn

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SvnTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the svn state
    """

    def setup_loader_modules(self):
        return {svn: {}}

    def test_latest(self):
        """
            Checkout or update the working directory to
            the latest revision from the remote repository.
        """
        mock = MagicMock(return_value=True)
        with patch.object(svn, "_fail", mock):
            self.assertTrue(svn.latest("salt"))

        mock = MagicMock(side_effect=[True, False, False, False])
        with patch.object(os.path, "exists", mock):
            mock = MagicMock(return_value=False)
            with patch.object(os.path, "isdir", mock):
                with patch.object(svn, "_fail", mock):
                    self.assertFalse(svn.latest("salt", "c://salt"))

            with patch.dict(svn.__opts__, {"test": True}):
                mock = MagicMock(return_value=["salt"])
                with patch.object(svn, "_neutral_test", mock):
                    self.assertListEqual(svn.latest("salt", "c://salt"), ["salt"])

                mock = MagicMock(side_effect=[False, True])
                with patch.object(os.path, "exists", mock):
                    mock = MagicMock(return_value=True)
                    info_mock = MagicMock(return_value=[{"Revision": "mocked"}])
                    with patch.dict(
                        svn.__salt__, {"svn.diff": mock, "svn.info": info_mock}
                    ):
                        mock = MagicMock(return_value=["Dude"])
                        with patch.object(svn, "_neutral_test", mock):
                            self.assertListEqual(
                                svn.latest("salt", "c://salt"), ["Dude"]
                            )

            with patch.dict(svn.__opts__, {"test": False}):
                mock = MagicMock(return_value=[{"Revision": "a"}])
                with patch.dict(svn.__salt__, {"svn.info": mock}):
                    mock = MagicMock(return_value=True)
                    with patch.dict(svn.__salt__, {"svn.update": mock}):
                        self.assertDictEqual(
                            svn.latest("salt", "c://salt"),
                            {
                                "changes": {},
                                "comment": True,
                                "name": "salt",
                                "result": True,
                            },
                        )

    def test_export(self):
        """
            Test to export a file or directory from an SVN repository
        """
        mock = MagicMock(return_value=True)
        with patch.object(svn, "_fail", mock):
            self.assertTrue(svn.export("salt"))

        mock = MagicMock(side_effect=[True, False, False, False])
        with patch.object(os.path, "exists", mock):
            mock = MagicMock(return_value=False)
            with patch.object(os.path, "isdir", mock):
                with patch.object(svn, "_fail", mock):
                    self.assertFalse(svn.export("salt", "c://salt"))

            with patch.dict(svn.__opts__, {"test": True}):
                mock = MagicMock(return_value=["salt"])
                with patch.object(svn, "_neutral_test", mock):
                    self.assertListEqual(svn.export("salt", "c://salt"), ["salt"])

                mock = MagicMock(side_effect=[False, True])
                with patch.object(os.path, "exists", mock):
                    mock = MagicMock(return_value=True)
                    with patch.dict(svn.__salt__, {"svn.list": mock}):
                        mock = MagicMock(return_value=["Dude"])
                        with patch.object(svn, "_neutral_test", mock):
                            self.assertListEqual(
                                svn.export("salt", "c://salt"), ["Dude"]
                            )

            with patch.dict(svn.__opts__, {"test": False}):
                mock = MagicMock(return_value=True)
                with patch.dict(svn.__salt__, {"svn.export": mock}):
                    self.assertDictEqual(
                        svn.export("salt", "c://salt"),
                        {
                            "changes": {
                                "new": "salt",
                                "comment": "salt was Exported to c://salt",
                            },
                            "comment": True,
                            "name": "salt",
                            "result": True,
                        },
                    )

    def test_dirty(self):
        """
            Test to determine if the working directory has been changed.
        """
        mock = MagicMock(return_value=True)
        with patch.object(svn, "_fail", mock):
            self.assertTrue(svn.dirty("salt", "c://salt"))



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import random
import string

# Import Salt libs
import salt.config
import salt.loader
import salt.states.boto_iot as boto_iot

# Import 3rd-party libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
from salt.utils.versions import LooseVersion
from tests.support.helpers import slowTest

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module,unused-import
from tests.unit.modules.test_boto_iot import BotoIoTTestCaseMixin

# Import test suite libs


try:
    import boto
    import boto3
    from botocore.exceptions import ClientError
    from botocore import __version__ as found_botocore_version

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# pylint: enable=import-error,no-name-in-module,unused-import

# the boto_iot module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = "1.2.1"
required_botocore_version = "1.4.41"

log = logging.getLogger(__name__)


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    elif LooseVersion(found_botocore_version) < LooseVersion(required_botocore_version):
        return False
    else:
        return True


if _has_required_boto():
    region = "us-east-1"
    access_key = "GKTADJGHEIQSXMKKRBJ08H"
    secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
    conn_parameters = {
        "region": region,
        "key": access_key,
        "keyid": secret_key,
        "profile": {},
    }
    error_message = (
        "An error occurred (101) when calling the {0} operation: Test-defined error"
    )
    not_found_error = ClientError(
        {
            "Error": {
                "Code": "ResourceNotFoundException",
                "Message": "Test-defined error",
            }
        },
        "msg",
    )
    topic_rule_not_found_error = ClientError(
        {"Error": {"Code": "UnauthorizedException", "Message": "Test-defined error"}},
        "msg",
    )
    error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
    policy_ret = dict(
        policyName="testpolicy",
        policyDocument='{"Version": "2012-10-17", "Statement": [{"Action": ["iot:Publish"], "Resource": ["*"], "Effect": "Allow"}]}',
        policyArn="arn:aws:iot:us-east-1:123456:policy/my_policy",
        policyVersionId=1,
        defaultVersionId=1,
    )
    topic_rule_ret = dict(
        ruleName="testrule",
        sql="SELECT * FROM 'iot/test'",
        description="topic rule description",
        createdAt="1970-01-01",
        actions=[{"iot": {"functionArn": "arn:aws:::function"}}],
        ruleDisabled=True,
    )
    principal = "arn:aws:iot:us-east-1:1234:cert/21fc104aaaf6043f5756c1b57bda84ea8395904c43f28517799b19e4c42514"

    thing_type_name = "test_thing_type"
    thing_type_desc = "test_thing_type_desc"
    thing_type_attr_1 = "test_thing_type_search_attr_1"
    thing_type_ret = dict(
        thingTypeName=thing_type_name,
        thingTypeProperties=dict(
            thingTypeDescription=thing_type_desc,
            searchableAttributes=[thing_type_attr_1],
        ),
        thingTypeMetadata=dict(
            deprecated=False, creationDate="2010-08-01 15:54:49.699000+00:00"
        ),
    )
    deprecated_thing_type_ret = dict(
        thingTypeName=thing_type_name,
        thingTypeProperties=dict(
            thingTypeDescription=thing_type_desc,
            searchableAttributes=[thing_type_attr_1],
        ),
        thingTypeMetadata=dict(
            deprecated=True,
            creationDate="2010-08-01 15:54:49.699000+00:00",
            deprecationDate="2010-08-02 15:54:49.699000+00:00",
        ),
    )
    thing_type_arn = "test_thing_type_arn"
    create_thing_type_ret = dict(
        thingTypeName=thing_type_name, thingTypeArn=thing_type_arn
    )


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than"
    " or equal to version {0}".format(required_boto3_version),
)
class BotoIoTStateTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        ctx = {}
        utils = salt.loader.utils(
            self.opts,
            whitelist=["boto3", "args", "systemd", "path", "platform"],
            context=ctx,
        )
        serializers = salt.loader.serializers(self.opts)
        self.funcs = funcs = salt.loader.minion_mods(
            self.opts, context=ctx, utils=utils, whitelist=["boto_iot"]
        )
        self.salt_states = salt.loader.states(
            opts=self.opts,
            functions=funcs,
            utils=utils,
            whitelist=["boto_iot"],
            serializers=serializers,
        )
        return {
            boto_iot: {
                "__opts__": self.opts,
                "__salt__": funcs,
                "__utils__": utils,
                "__states__": self.salt_states,
                "__serializers__": serializers,
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        cls.opts["grains"] = salt.loader.grains(cls.opts)

    @classmethod
    def tearDownClass(cls):
        del cls.opts

    def setUp(self):
        self.addCleanup(delattr, self, "funcs")
        self.addCleanup(delattr, self, "salt_states")
        # Set up MagicMock to replace the boto3 session
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        self.addCleanup(delattr, self, "conn")
        session_instance.client.return_value = self.conn


class BotoIoTThingTypeTestCase(BotoIoTStateTestCaseBase, BotoIoTTestCaseMixin):
    """
    TestCase for salt.modules.boto_iot state.module
    """

    def test_present_when_thing_type_does_not_exist(self):
        """
        tests present on a thing type that does not exist.
        """
        self.conn.describe_thing_type.side_effect = [not_found_error, thing_type_ret]
        self.conn.create_thing_type.return_value = create_thing_type_ret
        result = self.salt_states["boto_iot.thing_type_present"](
            "thing type present",
            thingTypeName=thing_type_name,
            thingTypeDescription=thing_type_desc,
            searchableAttributesList=[thing_type_attr_1],
            **conn_parameters
        )
        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"]["new"]["thing_type"]["thingTypeName"], thing_type_name
        )

    def test_present_when_thing_type_exists(self):
        self.conn.describe_thing_type.return_value = thing_type_ret
        result = self.salt_states["boto_iot.thing_type_present"](
            "thing type present",
            thingTypeName=thing_type_name,
            thingTypeDescription=thing_type_desc,
            searchableAttributesList=[thing_type_attr_1],
            **conn_parameters
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})
        self.assertTrue(self.conn.create_thing_type.call_count == 0)

    def test_present_with_failure(self):
        self.conn.describe_thing_type.side_effect = [not_found_error, thing_type_ret]
        self.conn.create_thing_type.side_effect = ClientError(
            error_content, "create_thing_type"
        )
        result = self.salt_states["boto_iot.thing_type_present"](
            "thing type present",
            thingTypeName=thing_type_name,
            thingTypeDescription=thing_type_desc,
            searchableAttributesList=[thing_type_attr_1],
            **conn_parameters
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_absent_when_thing_type_does_not_exist(self):
        """
        Tests absent on a thing type does not exist
        """
        self.conn.describe_thing_type.side_effect = not_found_error
        result = self.salt_states["boto_iot.thing_type_absent"](
            "test", "mythingtype", **conn_parameters
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    @slowTest
    def test_absent_when_thing_type_exists(self):
        """
        Tests absent on a thing type
        """
        self.conn.describe_thing_type.return_value = deprecated_thing_type_ret
        result = self.salt_states["boto_iot.thing_type_absent"](
            "test", thing_type_name, **conn_parameters
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["thing_type"], None)
        self.assertTrue(self.conn.deprecate_thing_type.call_count == 0)

    def test_absent_with_deprecate_failure(self):
        self.conn.describe_thing_type.return_value = thing_type_ret
        self.conn.deprecate_thing_type.side_effect = ClientError(
            error_content, "deprecate_thing_type"
        )
        result = self.salt_states["boto_iot.thing_type_absent"](
            "test", thing_type_name, **conn_parameters
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])
        self.assertTrue("deprecate_thing_type" in result["comment"])
        self.assertTrue(self.conn.delete_thing_type.call_count == 0)

    def test_absent_with_delete_failure(self):
        self.conn.describe_thing_type.return_value = deprecated_thing_type_ret
        self.conn.delete_thing_type.side_effect = ClientError(
            error_content, "delete_thing_type"
        )
        result = self.salt_states["boto_iot.thing_type_absent"](
            "test", thing_type_name, **conn_parameters
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])
        self.assertTrue("delete_thing_type" in result["comment"])
        self.assertTrue(self.conn.deprecate_thing_type.call_count == 0)


class BotoIoTPolicyTestCase(BotoIoTStateTestCaseBase, BotoIoTTestCaseMixin):
    """
    TestCase for salt.modules.boto_iot state.module
    """

    def test_present_when_policy_does_not_exist(self):
        """
        Tests present on a policy that does not exist.
        """
        self.conn.get_policy.side_effect = [not_found_error, policy_ret]
        self.conn.create_policy.return_value = policy_ret
        result = self.salt_states["boto_iot.policy_present"](
            "policy present",
            policyName=policy_ret["policyName"],
            policyDocument=policy_ret["policyDocument"],
        )

        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"]["new"]["policy"]["policyName"], policy_ret["policyName"]
        )

    def test_present_when_policy_exists(self):
        self.conn.get_policy.return_value = policy_ret
        self.conn.create_policy_version.return_value = policy_ret
        result = self.salt_states["boto_iot.policy_present"](
            "policy present",
            policyName=policy_ret["policyName"],
            policyDocument=policy_ret["policyDocument"],
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_present_with_failure(self):
        self.conn.get_policy.side_effect = [not_found_error, policy_ret]
        self.conn.create_policy.side_effect = ClientError(
            error_content, "create_policy"
        )
        result = self.salt_states["boto_iot.policy_present"](
            "policy present",
            policyName=policy_ret["policyName"],
            policyDocument=policy_ret["policyDocument"],
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_absent_when_policy_does_not_exist(self):
        """
        Tests absent on a policy that does not exist.
        """
        self.conn.get_policy.side_effect = not_found_error
        result = self.salt_states["boto_iot.policy_absent"]("test", "mypolicy")
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_absent_when_policy_exists(self):
        self.conn.get_policy.return_value = policy_ret
        self.conn.list_policy_versions.return_value = {"policyVersions": []}
        result = self.salt_states["boto_iot.policy_absent"](
            "test", policy_ret["policyName"]
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["policy"], None)

    def test_absent_with_failure(self):
        self.conn.get_policy.return_value = policy_ret
        self.conn.list_policy_versions.return_value = {"policyVersions": []}
        self.conn.delete_policy.side_effect = ClientError(
            error_content, "delete_policy"
        )
        result = self.salt_states["boto_iot.policy_absent"](
            "test", policy_ret["policyName"]
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_attached_when_policy_not_attached(self):
        """
        Tests attached on a policy that is not attached.
        """
        self.conn.list_principal_policies.return_value = {"policies": []}
        result = self.salt_states["boto_iot.policy_attached"](
            "test", "myfunc", principal
        )
        self.assertTrue(result["result"])
        self.assertTrue(result["changes"]["new"]["attached"])

    def test_attached_when_policy_attached(self):
        """
        Tests attached on a policy that is attached.
        """
        self.conn.list_principal_policies.return_value = {"policies": [policy_ret]}
        result = self.salt_states["boto_iot.policy_attached"](
            "test", policy_ret["policyName"], principal
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_attached_with_failure(self):
        """
        Tests attached on a policy that is attached.
        """
        self.conn.list_principal_policies.return_value = {"policies": []}
        self.conn.attach_principal_policy.side_effect = ClientError(
            error_content, "attach_principal_policy"
        )
        result = self.salt_states["boto_iot.policy_attached"](
            "test", policy_ret["policyName"], principal
        )
        self.assertFalse(result["result"])
        self.assertEqual(result["changes"], {})

    def test_detached_when_policy_not_detached(self):
        """
        Tests detached on a policy that is not detached.
        """
        self.conn.list_principal_policies.return_value = {"policies": [policy_ret]}
        result = self.salt_states["boto_iot.policy_detached"](
            "test", policy_ret["policyName"], principal
        )
        self.assertTrue(result["result"])
        log.warning(result)
        self.assertFalse(result["changes"]["new"]["attached"])

    def test_detached_when_policy_detached(self):
        """
        Tests detached on a policy that is detached.
        """
        self.conn.list_principal_policies.return_value = {"policies": []}
        result = self.salt_states["boto_iot.policy_detached"](
            "test", policy_ret["policyName"], principal
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_detached_with_failure(self):
        """
        Tests detached on a policy that is detached.
        """
        self.conn.list_principal_policies.return_value = {"policies": [policy_ret]}
        self.conn.detach_principal_policy.side_effect = ClientError(
            error_content, "detach_principal_policy"
        )
        result = self.salt_states["boto_iot.policy_detached"](
            "test", policy_ret["policyName"], principal
        )
        self.assertFalse(result["result"])
        self.assertEqual(result["changes"], {})


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than"
    " or equal to version {0}.  The botocore"
    " module must be greater than or equal to"
    " version {1}.".format(required_boto3_version, required_botocore_version),
)
class BotoIoTTopicRuleTestCase(BotoIoTStateTestCaseBase, BotoIoTTestCaseMixin):
    """
    TestCase for salt.modules.boto_iot state.module rules
    """

    def test_present_when_topic_rule_does_not_exist(self):
        """
        Tests present on a topic_rule that does not exist.
        """
        self.conn.get_topic_rule.side_effect = [
            topic_rule_not_found_error,
            {"rule": topic_rule_ret},
        ]
        self.conn.create_topic_rule.return_value = {"created": True}
        result = self.salt_states["boto_iot.topic_rule_present"](
            "topic rule present",
            ruleName=topic_rule_ret["ruleName"],
            sql=topic_rule_ret["sql"],
            description=topic_rule_ret["description"],
            actions=topic_rule_ret["actions"],
            ruleDisabled=topic_rule_ret["ruleDisabled"],
        )

        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"]["new"]["rule"]["ruleName"], topic_rule_ret["ruleName"]
        )

    def test_present_when_policy_exists(self):
        self.conn.get_topic_rule.return_value = {"rule": topic_rule_ret}
        self.conn.create_topic_rule.return_value = {"created": True}
        result = self.salt_states["boto_iot.topic_rule_present"](
            "topic rule present",
            ruleName=topic_rule_ret["ruleName"],
            sql=topic_rule_ret["sql"],
            description=topic_rule_ret["description"],
            actions=topic_rule_ret["actions"],
            ruleDisabled=topic_rule_ret["ruleDisabled"],
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_present_with_failure(self):
        self.conn.get_topic_rule.side_effect = [
            topic_rule_not_found_error,
            {"rule": topic_rule_ret},
        ]
        self.conn.create_topic_rule.side_effect = ClientError(
            error_content, "create_topic_rule"
        )
        result = self.salt_states["boto_iot.topic_rule_present"](
            "topic rule present",
            ruleName=topic_rule_ret["ruleName"],
            sql=topic_rule_ret["sql"],
            description=topic_rule_ret["description"],
            actions=topic_rule_ret["actions"],
            ruleDisabled=topic_rule_ret["ruleDisabled"],
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_absent_when_topic_rule_does_not_exist(self):
        """
        Tests absent on a topic rule that does not exist.
        """
        self.conn.get_topic_rule.side_effect = topic_rule_not_found_error
        result = self.salt_states["boto_iot.topic_rule_absent"]("test", "myrule")
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_absent_when_topic_rule_exists(self):
        self.conn.get_topic_rule.return_value = topic_rule_ret
        result = self.salt_states["boto_iot.topic_rule_absent"](
            "test", topic_rule_ret["ruleName"]
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["rule"], None)

    def test_absent_with_failure(self):
        self.conn.get_topic_rule.return_value = topic_rule_ret
        self.conn.delete_topic_rule.side_effect = ClientError(
            error_content, "delete_topic_rule"
        )
        result = self.salt_states["boto_iot.topic_rule_absent"](
            "test", topic_rule_ret["ruleName"]
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

import salt.modules.environ as envmodule
import salt.modules.reg

# Import salt libs
import salt.states.environ as envstate
import salt.utils.platform

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


class TestEnvironState(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        loader_globals = {
            "__env__": "base",
            "__opts__": {"test": False},
            "__salt__": {"environ.setenv": envmodule.setenv},
        }
        return {envstate: loader_globals, envmodule: loader_globals}

    def setUp(self):
        patcher = patch.dict(os.environ, {"INITIAL": "initial"}, clear=True)
        patcher.start()

        def reset_environ(patcher):
            patcher.stop()

        self.addCleanup(reset_environ, patcher)

    def test_setenv(self):
        """
        test that a subsequent calls of setenv changes nothing
        """
        ret = envstate.setenv("test", "value")
        self.assertEqual(ret["changes"], {"test": "value"})

        ret = envstate.setenv("test", "other")
        self.assertEqual(ret["changes"], {"test": "other"})

        # once again with the same value
        ret = envstate.setenv("test", "other")
        self.assertEqual(ret["changes"], {})

    @skipIf(not salt.utils.platform.is_windows(), "Windows only")
    def test_setenv_permanent(self):
        """
        test that we can set permanent environment variables (requires pywin32)
        """
        with patch.dict(
            envmodule.__utils__,
            {
                "reg.set_value": MagicMock(),
                "reg.delete_value": MagicMock(),
                "win_functions.broadcast_setting_change": MagicMock(),
            },
        ):
            ret = envstate.setenv("test", "value", permanent=True)
            self.assertEqual(ret["changes"], {"test": "value"})
            envmodule.__utils__["reg.set_value"].assert_called_with(
                "HKCU", "Environment", "test", "value"
            )

            ret = envstate.setenv("test", False, false_unsets=True, permanent=True)
            self.assertEqual(ret["changes"], {"test": None})
            envmodule.__utils__["reg.delete_value"].assert_called_with(
                "HKCU", "Environment", "test"
            )

    def test_setenv_dict(self):
        """
        test that setenv can be invoked with dict
        """
        ret = envstate.setenv("notimportant", {"test": "value"})
        self.assertEqual(ret["changes"], {"test": "value"})

    def test_setenv_int(self):
        """
        test that setenv can not be invoked with int
        (actually it's anything other than strings and dict)
        """
        ret = envstate.setenv("test", 1)
        self.assertEqual(ret["result"], False)

    def test_setenv_unset(self):
        """
        test that ``false_unsets`` option removes variable from environment
        """
        ret = envstate.setenv("test", "value")
        self.assertEqual(ret["changes"], {"test": "value"})

        ret = envstate.setenv("notimportant", {"test": False}, false_unsets=True)
        self.assertEqual(ret["changes"], {"test": None})
        self.assertEqual(envstate.os.environ, {"INITIAL": "initial"})

    def test_setenv_clearall(self):
        """
        test that ``clear_all`` option sets other values to ''
        """
        ret = envstate.setenv("test", "value", clear_all=True)
        self.assertEqual(ret["changes"], {"test": "value", "INITIAL": ""})
        if salt.utils.platform.is_windows():
            self.assertEqual(envstate.os.environ, {"TEST": "value", "INITIAL": ""})
        else:
            self.assertEqual(envstate.os.environ, {"test": "value", "INITIAL": ""})

    def test_setenv_clearall_with_unset(self):
        """
        test that ``clear_all`` option combined with ``false_unsets``
        unsets other values from environment
        """
        ret = envstate.setenv("test", "value", false_unsets=True, clear_all=True)
        self.assertEqual(ret["changes"], {"test": "value", "INITIAL": None})
        if salt.utils.platform.is_windows():
            self.assertEqual(envstate.os.environ, {"TEST": "value"})
        else:
            self.assertEqual(envstate.os.environ, {"test": "value"})

    def test_setenv_unset_multi(self):
        """
        test basically same things that above tests but with multiple values passed
        """
        ret = envstate.setenv("notimportant", {"foo": "bar"})
        self.assertEqual(ret["changes"], {"foo": "bar"})

        with patch.dict(envstate.__utils__, {"reg.read_value": MagicMock()}):
            ret = envstate.setenv(
                "notimportant", {"test": False, "foo": "baz"}, false_unsets=True
            )
        self.assertEqual(ret["changes"], {"test": None, "foo": "baz"})
        if salt.utils.platform.is_windows():
            self.assertEqual(envstate.os.environ, {"INITIAL": "initial", "FOO": "baz"})
        else:
            self.assertEqual(envstate.os.environ, {"INITIAL": "initial", "foo": "baz"})

        with patch.dict(envstate.__utils__, {"reg.read_value": MagicMock()}):
            ret = envstate.setenv("notimportant", {"test": False, "foo": "bax"})
        self.assertEqual(ret["changes"], {"test": "", "foo": "bax"})
        if salt.utils.platform.is_windows():
            self.assertEqual(
                envstate.os.environ, {"INITIAL": "initial", "FOO": "bax", "TEST": ""}
            )
        else:
            self.assertEqual(
                envstate.os.environ, {"INITIAL": "initial", "foo": "bax", "test": ""}
            )

    def test_setenv_test_mode(self):
        """
        test that imitating action returns good values
        """
        with patch.dict(envstate.__opts__, {"test": True}):
            ret = envstate.setenv("test", "value")
            self.assertEqual(ret["changes"], {"test": "value"})
            ret = envstate.setenv("INITIAL", "initial")
            self.assertEqual(ret["changes"], {})



# -*- coding: utf-8 -*-
"""
Utility functions for use with or in SLS files
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import textwrap

# Import Salt libs
import salt.exceptions
import salt.loader
import salt.template
import salt.utils.args
import salt.utils.dictupdate


def update(dest, upd, recursive_update=True, merge_lists=False):
    """
    Merge ``upd`` recursively into ``dest``

    If ``merge_lists=True``, will aggregate list object types instead of
    replacing. This behavior is only activated when ``recursive_update=True``.

    CLI Example:

    .. code-block:: shell

        salt '*' slsutil.update '{foo: Foo}' '{bar: Bar}'

    """
    return salt.utils.dictupdate.update(dest, upd, recursive_update, merge_lists)


def merge(obj_a, obj_b, strategy="smart", renderer="yaml", merge_lists=False):
    """
    Merge a data structure into another by choosing a merge strategy

    Strategies:

    * aggregate
    * list
    * overwrite
    * recurse
    * smart

    CLI Example:

    .. code-block:: shell

        salt '*' slsutil.merge '{foo: Foo}' '{bar: Bar}'
    """
    return salt.utils.dictupdate.merge(obj_a, obj_b, strategy, renderer, merge_lists)


def merge_all(lst, strategy="smart", renderer="yaml", merge_lists=False):
    """
    .. versionadded:: 2019.2.0

    Merge a list of objects into each other in order

    :type lst: Iterable
    :param lst: List of objects to be merged.

    :type strategy: String
    :param strategy: Merge strategy. See utils.dictupdate.

    :type renderer: String
    :param renderer:
        Renderer type. Used to determine strategy when strategy is 'smart'.

    :type merge_lists: Bool
    :param merge_lists: Defines whether to merge embedded object lists.

    CLI Example:

    .. code-block:: shell

        $ salt-call --output=txt slsutil.merge_all '[{foo: Foo}, {foo: Bar}]'
        local: {u'foo': u'Bar'}
    """

    ret = {}
    for obj in lst:
        ret = salt.utils.dictupdate.merge(ret, obj, strategy, renderer, merge_lists)

    return ret


def renderer(path=None, string=None, default_renderer="jinja|yaml", **kwargs):
    """
    Parse a string or file through Salt's renderer system

    .. versionchanged:: 2018.3.0
       Add support for Salt fileserver URIs.

    This is an open-ended function and can be used for a variety of tasks. It
    makes use of Salt's "renderer pipes" system to run a string or file through
    a pipe of any of the loaded renderer modules.

    :param path: The path to a file on Salt's fileserver (any URIs supported by
        :py:func:`cp.get_url <salt.modules.cp.get_url>`) or on the local file
        system.
    :param string: An inline string to be used as the file to send through the
        renderer system. Note, not all renderer modules can work with strings;
        the 'py' renderer requires a file, for example.
    :param default_renderer: The renderer pipe to send the file through; this
        is overridden by a "she-bang" at the top of the file.
    :param kwargs: Keyword args to pass to Salt's compile_template() function.

    Keep in mind the goal of each renderer when choosing a render-pipe; for
    example, the Jinja renderer processes a text file and produces a string,
    however the YAML renderer processes a text file and produces a data
    structure.

    One possible use is to allow writing "map files", as are commonly seen in
    Salt formulas, but without tying the renderer of the map file to the
    renderer used in the other sls files. In other words, a map file could use
    the Python renderer and still be included and used by an sls file that uses
    the default 'jinja|yaml' renderer.

    For example, the two following map files produce identical results but one
    is written using the normal 'jinja|yaml' and the other is using 'py':

    .. code-block:: jinja

        #!jinja|yaml
        {% set apache = salt.grains.filter_by({
            ...normal jinja map file here...
        }, merge=salt.pillar.get('apache:lookup')) %}
        {{ apache | yaml() }}

    .. code-block:: python

        #!py
        def run():
            apache = __salt__.grains.filter_by({
                ...normal map here but as a python dict...
            }, merge=__salt__.pillar.get('apache:lookup'))
            return apache

    Regardless of which of the above map files is used, it can be accessed from
    any other sls file by calling this function. The following is a usage
    example in Jinja:

    .. code-block:: jinja

        {% set apache = salt.slsutil.renderer('map.sls') %}

    CLI Example:

    .. code-block:: bash

        salt '*' slsutil.renderer salt://path/to/file
        salt '*' slsutil.renderer /path/to/file
        salt '*' slsutil.renderer /path/to/file.jinja 'jinja'
        salt '*' slsutil.renderer /path/to/file.sls 'jinja|yaml'
        salt '*' slsutil.renderer string='Inline template! {{ saltenv }}'
        salt '*' slsutil.renderer string='Hello, {{ name }}.' name='world'
    """
    if not path and not string:
        raise salt.exceptions.SaltInvocationError("Must pass either path or string")

    renderers = salt.loader.render(__opts__, __salt__)

    if path:
        path_or_string = __salt__["cp.get_url"](
            path, saltenv=kwargs.get("saltenv", "base")
        )
    elif string:
        path_or_string = ":string:"
        kwargs["input_data"] = string

    ret = salt.template.compile_template(
        path_or_string,
        renderers,
        default_renderer,
        __opts__["renderer_blacklist"],
        __opts__["renderer_whitelist"],
        **kwargs
    )
    return ret.read() if __utils__["stringio.is_readable"](ret) else ret


def _get_serialize_fn(serializer, fn_name):
    serializers = salt.loader.serializers(__opts__)
    fns = getattr(serializers, serializer, None)
    fn = getattr(fns, fn_name, None)

    if not fns:
        raise salt.exceptions.CommandExecutionError(
            "Serializer '{0}' not found.".format(serializer)
        )

    if not fn:
        raise salt.exceptions.CommandExecutionError(
            "Serializer '{0}' does not implement {1}.".format(serializer, fn_name)
        )

    return fn


def serialize(serializer, obj, **mod_kwargs):
    """
    Serialize a Python object using one of the available
    :ref:`all-salt.serializers`.

    CLI Example:

    .. code-block:: bash

        salt '*' --no-parse=obj slsutil.serialize 'json' obj="{'foo': 'Foo!'}

    Jinja Example:

    .. code-block:: jinja

        {% set json_string = salt.slsutil.serialize('json',
            {'foo': 'Foo!'}) %}
    """
    kwargs = salt.utils.args.clean_kwargs(**mod_kwargs)
    return _get_serialize_fn(serializer, "serialize")(obj, **kwargs)


def deserialize(serializer, stream_or_string, **mod_kwargs):
    """
    Deserialize a Python object using one of the available
    :ref:`all-salt.serializers`.

    CLI Example:

    .. code-block:: bash

        salt '*' slsutil.deserialize 'json' '{"foo": "Foo!"}'
        salt '*' --no-parse=stream_or_string slsutil.deserialize 'json' \\
            stream_or_string='{"foo": "Foo!"}'

    Jinja Example:

    .. code-block:: jinja

        {% set python_object = salt.slsutil.deserialize('json',
            '{"foo": "Foo!"}') %}
    """
    kwargs = salt.utils.args.clean_kwargs(**mod_kwargs)
    return _get_serialize_fn(serializer, "deserialize")(stream_or_string, **kwargs)


def banner(
    width=72,
    commentchar="#",
    borderchar="#",
    blockstart=None,
    blockend=None,
    title=None,
    text=None,
    newline=False,
):
    """
    Create a standardized comment block to include in a templated file.

    A common technique in configuration management is to include a comment
    block in managed files, warning users not to modify the file. This
    function simplifies and standardizes those comment blocks.

    :param width: The width, in characters, of the banner. Default is 72.
    :param commentchar: The character to be used in the starting position of
        each line. This value should be set to a valid line comment character
        for the syntax of the file in which the banner is being inserted.
        Multiple character sequences, like '//' are supported.
        If the file's syntax does not support line comments (such as XML),
        use the ``blockstart`` and ``blockend`` options.
    :param borderchar: The character to use in the top and bottom border of
        the comment box. Must be a single character.
    :param blockstart: The character sequence to use at the beginning of a
        block comment. Should be used in conjunction with ``blockend``
    :param blockend: The character sequence to use at the end of a
        block comment. Should be used in conjunction with ``blockstart``
    :param title: The first field of the comment block. This field appears
        centered at the top of the box.
    :param text: The second filed of the comment block. This field appears
        left-justifed at the bottom of the box.
    :param newline: Boolean value to indicate whether the comment block should
        end with a newline. Default is ``False``.

    **Example 1 - the default banner:**

    .. code-block:: jinja

        {{ salt['slsutil.banner']() }}

    .. code-block:: none

        ########################################################################
        #                                                                      #
        #              THIS FILE IS MANAGED BY SALT - DO NOT EDIT              #
        #                                                                      #
        # The contents of this file are managed by Salt. Any changes to this   #
        # file may be overwritten automatically and without warning.           #
        ########################################################################

    **Example 2 - a Javadoc-style banner:**

    .. code-block:: jinja

        {{ salt['slsutil.banner'](commentchar=' *', borderchar='*', blockstart='/**', blockend=' */') }}

    .. code-block:: none

        /**
         ***********************************************************************
         *                                                                     *
         *              THIS FILE IS MANAGED BY SALT - DO NOT EDIT             *
         *                                                                     *
         * The contents of this file are managed by Salt. Any changes to this  *
         * file may be overwritten automatically and without warning.          *
         ***********************************************************************
         */

    **Example 3 - custom text:**

    .. code-block:: jinja

        {{ set copyright='This file may not be copied or distributed without permission of SaltStack, Inc.' }}
        {{ salt['slsutil.banner'](title='Copyright 2019 SaltStack, Inc.', text=copyright, width=60) }}

    .. code-block:: none

        ############################################################
        #                                                          #
        #              Copyright 2019 SaltStack, Inc.              #
        #                                                          #
        # This file may not be copied or distributed without       #
        # permission of SaltStack, Inc.                            #
        ############################################################

    """

    if title is None:
        title = "THIS FILE IS MANAGED BY SALT - DO NOT EDIT"

    if text is None:
        text = (
            "The contents of this file are managed by Salt. "
            "Any changes to this file may be overwritten "
            "automatically and without warning."
        )

    # Set up some typesetting variables
    ledge = commentchar.rstrip()
    redge = commentchar.strip()
    lgutter = ledge + " "
    rgutter = " " + redge
    textwidth = width - len(lgutter) - len(rgutter)

    # Check the width
    if textwidth <= 0:
        raise salt.exceptions.ArgumentValueError("Width is too small to render banner")

    # Define the static elements
    border_line = (
        commentchar + borderchar[:1] * (width - len(ledge) - len(redge)) + redge
    )
    spacer_line = commentchar + " " * (width - len(commentchar) * 2) + commentchar

    # Create the banner
    wrapper = textwrap.TextWrapper(width=textwidth)
    block = list()
    if blockstart is not None:
        block.append(blockstart)
    block.append(border_line)
    block.append(spacer_line)
    for line in wrapper.wrap(title):
        block.append(lgutter + line.center(textwidth) + rgutter)
    block.append(spacer_line)
    for line in wrapper.wrap(text):
        block.append(lgutter + line + " " * (textwidth - len(line)) + rgutter)
    block.append(border_line)
    if blockend is not None:
        block.append(blockend)

    # Convert list to multi-line string
    result = os.linesep.join(block)

    # Add a newline character to the end of the banner
    if newline:
        return result + os.linesep

    return result


def boolstr(value, true="true", false="false"):
    """
    Convert a boolean value into a string. This function is
    intended to be used from within file templates to provide
    an easy way to take boolean values stored in Pillars or
    Grains, and write them out in the apprpriate syntax for
    a particular file template.

    :param value: The boolean value to be converted
    :param true: The value to return if ``value`` is ``True``
    :param false: The value to return if ``value`` is ``False``

    In this example, a pillar named ``smtp:encrypted`` stores a boolean
    value, but the template that uses that value needs ``yes`` or ``no``
    to be written, based on the boolean value.

    *Note: this is written on two lines for clarity. The same result
    could be achieved in one line.*

    .. code-block:: jinja

        {% set encrypted = salt[pillar.get]('smtp:encrypted', false) %}
        use_tls: {{ salt['slsutil.boolstr'](encrypted, 'yes', 'no') }}

    Result (assuming the value is ``True``):

    .. code-block:: none

        use_tls: yes

    """

    if value:
        return true

    return false



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import random
import string

# Import Salt libs
import salt.config
import salt.loader
import salt.states.boto_cloudwatch_event as boto_cloudwatch_event
from salt.ext.six.moves import range

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module
from tests.unit.modules.test_boto_cloudwatch_event import (
    BotoCloudWatchEventTestCaseMixin,
)

# pylint: disable=unused-import
# Import 3rd-party libs
try:
    import boto3
    from botocore.exceptions import ClientError

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False
# pylint: enable=unused-import


# pylint: enable=import-error,no-name-in-module

region = "us-east-1"
access_key = "GKTADJGHEIQSXMKKRBJ08H"
secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
conn_parameters = {
    "region": region,
    "key": access_key,
    "keyid": secret_key,
    "profile": {},
}
error_message = (
    "An error occurred (101) when calling the {0} operation: Test-defined error"
)
error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
if HAS_BOTO:
    not_found_error = ClientError(
        {
            "Error": {
                "Code": "ResourceNotFoundException",
                "Message": "Test-defined error",
            }
        },
        "msg",
    )
rule_name = "test_thing_type"
rule_desc = "test_thing_type_desc"
rule_sched = "rate(20 min)"
rule_arn = "arn:::::rule/arn"
rule_ret = dict(
    Arn=rule_arn,
    Description=rule_desc,
    EventPattern=None,
    Name=rule_name,
    RoleArn=None,
    ScheduleExpression=rule_sched,
    State="ENABLED",
)


log = logging.getLogger(__name__)


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    else:
        return True


class BotoCloudWatchEventStateTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        ctx = {}
        utils = salt.loader.utils(
            self.opts,
            whitelist=["boto3", "args", "systemd", "path", "platform"],
            context=ctx,
        )
        serializers = salt.loader.serializers(self.opts)
        self.funcs = funcs = salt.loader.minion_mods(
            self.opts, context=ctx, utils=utils, whitelist=["boto_cloudwatch_event"]
        )
        self.salt_states = salt.loader.states(
            opts=self.opts,
            functions=funcs,
            utils=utils,
            whitelist=["boto_cloudwatch_event"],
            serializers=serializers,
        )
        return {
            boto_cloudwatch_event: {
                "__opts__": self.opts,
                "__salt__": funcs,
                "__utils__": utils,
                "__states__": self.salt_states,
                "__serializers__": serializers,
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        cls.opts["grains"] = salt.loader.grains(cls.opts)

    @classmethod
    def tearDownClass(cls):
        del cls.opts

    def setUp(self):
        self.addCleanup(delattr, self, "funcs")
        self.addCleanup(delattr, self, "salt_states")
        # Set up MagicMock to replace the boto3 session
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        self.addCleanup(delattr, self, "conn")
        session_instance.client.return_value = self.conn


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
class BotoCloudWatchEventTestCase(
    BotoCloudWatchEventStateTestCaseBase, BotoCloudWatchEventTestCaseMixin
):
    def test_present_when_failing_to_describe_rule(self):
        """
        Tests exceptions when checking rule existence
        """
        self.conn.list_rules.side_effect = ClientError(
            error_content, "error on list rules"
        )
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("error on list rules" in result.get("comment", {}))

    def test_present_when_failing_to_create_a_new_rule(self):
        """
        Tests present on a rule name that doesn't exist and
        an error is thrown on creation.
        """
        self.conn.list_rules.return_value = {"Rules": []}
        self.conn.put_rule.side_effect = ClientError(error_content, "put_rule")
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("put_rule" in result.get("comment", ""))

    def test_present_when_failing_to_describe_the_new_rule(self):
        """
        Tests present on a rule name that doesn't exist and
        an error is thrown when adding targets.
        """
        self.conn.list_rules.return_value = {"Rules": []}
        self.conn.put_rule.return_value = rule_ret
        self.conn.describe_rule.side_effect = ClientError(
            error_content, "describe_rule"
        )
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("describe_rule" in result.get("comment", ""))

    def test_present_when_failing_to_create_a_new_rules_targets(self):
        """
        Tests present on a rule name that doesn't exist and
        an error is thrown when adding targets.
        """
        self.conn.list_rules.return_value = {"Rules": []}
        self.conn.put_rule.return_value = rule_ret
        self.conn.describe_rule.return_value = rule_ret
        self.conn.put_targets.side_effect = ClientError(error_content, "put_targets")
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("put_targets" in result.get("comment", ""))

    def test_present_when_rule_does_not_exist(self):
        """
        Tests the successful case of creating a new rule, and updating its
        targets
        """
        self.conn.list_rules.return_value = {"Rules": []}
        self.conn.put_rule.return_value = rule_ret
        self.conn.describe_rule.return_value = rule_ret
        self.conn.put_targets.return_value = {"FailedEntryCount": 0}
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), True)

    def test_present_when_failing_to_update_an_existing_rule(self):
        """
        Tests present on an existing rule where an error is thrown on updating the pool properties.
        """
        self.conn.list_rules.return_value = {"Rules": [rule_ret]}
        self.conn.describe_rule.side_effect = ClientError(
            error_content, "describe_rule"
        )
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("describe_rule" in result.get("comment", ""))

    def test_present_when_failing_to_get_targets(self):
        """
        Tests present on an existing rule where put_rule succeeded, but an error
        is thrown on getting targets
        """
        self.conn.list_rules.return_value = {"Rules": [rule_ret]}
        self.conn.put_rule.return_value = rule_ret
        self.conn.describe_rule.return_value = rule_ret
        self.conn.list_targets_by_rule.side_effect = ClientError(
            error_content, "list_targets"
        )
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("list_targets" in result.get("comment", ""))

    def test_present_when_failing_to_put_targets(self):
        """
        Tests present on an existing rule where put_rule succeeded, but an error
        is thrown on putting targets
        """
        self.conn.list_rules.return_value = {"Rules": []}
        self.conn.put_rule.return_value = rule_ret
        self.conn.describe_rule.return_value = rule_ret
        self.conn.list_targets.return_value = {"Targets": []}
        self.conn.put_targets.side_effect = ClientError(error_content, "put_targets")
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("put_targets" in result.get("comment", ""))

    def test_present_when_putting_targets(self):
        """
        Tests present on an existing rule where put_rule succeeded, and targets
        must be added
        """
        self.conn.list_rules.return_value = {"Rules": []}
        self.conn.put_rule.return_value = rule_ret
        self.conn.describe_rule.return_value = rule_ret
        self.conn.list_targets.return_value = {"Targets": []}
        self.conn.put_targets.return_value = {"FailedEntryCount": 0}
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), True)

    def test_present_when_removing_targets(self):
        """
        Tests present on an existing rule where put_rule succeeded, and targets
        must be removed
        """
        self.conn.list_rules.return_value = {"Rules": []}
        self.conn.put_rule.return_value = rule_ret
        self.conn.describe_rule.return_value = rule_ret
        self.conn.list_targets.return_value = {
            "Targets": [{"Id": "target1"}, {"Id": "target2"}]
        }
        self.conn.put_targets.return_value = {"FailedEntryCount": 0}
        result = self.salt_states["boto_cloudwatch_event.present"](
            name="test present",
            Name=rule_name,
            Description=rule_desc,
            ScheduleExpression=rule_sched,
            Targets=[{"Id": "target1", "Arn": "arn::::::*"}],
            **conn_parameters
        )
        self.assertEqual(result.get("result"), True)

    def test_absent_when_failing_to_describe_rule(self):
        """
        Tests exceptions when checking rule existence
        """
        self.conn.list_rules.side_effect = ClientError(
            error_content, "error on list rules"
        )
        result = self.salt_states["boto_cloudwatch_event.absent"](
            name="test present", Name=rule_name, **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("error on list rules" in result.get("comment", {}))

    def test_absent_when_rule_does_not_exist(self):
        """
        Tests absent on an non-existing rule
        """
        self.conn.list_rules.return_value = {"Rules": []}
        result = self.salt_states["boto_cloudwatch_event.absent"](
            name="test absent", Name=rule_name, **conn_parameters
        )
        self.assertEqual(result.get("result"), True)
        self.assertEqual(result["changes"], {})

    def test_absent_when_failing_to_list_targets(self):
        """
        Tests absent on an rule when the list_targets call fails
        """
        self.conn.list_rules.return_value = {"Rules": [rule_ret]}
        self.conn.list_targets_by_rule.side_effect = ClientError(
            error_content, "list_targets"
        )
        result = self.salt_states["boto_cloudwatch_event.absent"](
            name="test absent", Name=rule_name, **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("list_targets" in result.get("comment", ""))

    def test_absent_when_failing_to_remove_targets_exception(self):
        """
        Tests absent on an rule when the remove_targets call fails
        """
        self.conn.list_rules.return_value = {"Rules": [rule_ret]}
        self.conn.list_targets_by_rule.return_value = {"Targets": [{"Id": "target1"}]}
        self.conn.remove_targets.side_effect = ClientError(
            error_content, "remove_targets"
        )
        result = self.salt_states["boto_cloudwatch_event.absent"](
            name="test absent", Name=rule_name, **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("remove_targets" in result.get("comment", ""))

    def test_absent_when_failing_to_remove_targets_nonexception(self):
        """
        Tests absent on an rule when the remove_targets call fails
        """
        self.conn.list_rules.return_value = {"Rules": [rule_ret]}
        self.conn.list_targets_by_rule.return_value = {"Targets": [{"Id": "target1"}]}
        self.conn.remove_targets.return_value = {"FailedEntryCount": 1}
        result = self.salt_states["boto_cloudwatch_event.absent"](
            name="test absent", Name=rule_name, **conn_parameters
        )
        self.assertEqual(result.get("result"), False)

    def test_absent_when_failing_to_delete_rule(self):
        """
        Tests absent on an rule when the delete_rule call fails
        """
        self.conn.list_rules.return_value = {"Rules": [rule_ret]}
        self.conn.list_targets_by_rule.return_value = {"Targets": [{"Id": "target1"}]}
        self.conn.remove_targets.return_value = {"FailedEntryCount": 0}
        self.conn.delete_rule.side_effect = ClientError(error_content, "delete_rule")
        result = self.salt_states["boto_cloudwatch_event.absent"](
            name="test absent", Name=rule_name, **conn_parameters
        )
        self.assertEqual(result.get("result"), False)
        self.assertTrue("delete_rule" in result.get("comment", ""))

    def test_absent(self):
        """
        Tests absent on an rule
        """
        self.conn.list_rules.return_value = {"Rules": [rule_ret]}
        self.conn.list_targets_by_rule.return_value = {"Targets": [{"Id": "target1"}]}
        self.conn.remove_targets.return_value = {"FailedEntryCount": 0}
        result = self.salt_states["boto_cloudwatch_event.absent"](
            name="test absent", Name=rule_name, **conn_parameters
        )
        self.assertEqual(result.get("result"), True)



# coding: utf-8

# Python libs
from __future__ import absolute_import

import logging
from collections import namedtuple

# Salt libs
import salt.beacons.network_info as network_info
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase

log = logging.getLogger(__name__)

STUB_NET_IO_COUNTERS = {
    "eth0": namedtuple(
        "snetio",
        "bytes_sent bytes_recv \
                                           packets_sent packets_recv \
                                           errin errout \
                                           dropin \
                                           dropout",
    )(93662618, 914626664, 465694, 903802, 0, 0, 0, 0)
}


class NetworkInfoBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.network_info
    """

    def setup_loader_modules(self):
        return {network_info: {"__context__": {}, "__salt__": {}}}

    def test_non_list_config(self):
        config = {}

        ret = network_info.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for network_info beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = network_info.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

    def test_network_info_equal(self):
        with patch(
            "salt.utils.psutil_compat.net_io_counters",
            MagicMock(return_value=STUB_NET_IO_COUNTERS),
        ):
            config = [
                {
                    "interfaces": {
                        "eth0": {
                            "type": "equal",
                            "bytes_sent": 914626664,
                            "bytes_recv": 93662618,
                            "packets_sent": 465694,
                            "packets_recv": 903802,
                            "errin": 0,
                            "errout": 0,
                            "dropin": 0,
                            "dropout": 0,
                        }
                    }
                }
            ]

            ret = network_info.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            _expected_return = [
                {
                    "interface": "eth0",
                    "network_info": {
                        "bytes_recv": 914626664,
                        "bytes_sent": 93662618,
                        "dropin": 0,
                        "dropout": 0,
                        "errin": 0,
                        "errout": 0,
                        "packets_recv": 903802,
                        "packets_sent": 465694,
                    },
                }
            ]

            ret = network_info.beacon(config)
            self.assertEqual(ret, _expected_return)

    def test_network_info_greater_than(self):
        with patch(
            "salt.utils.psutil_compat.net_io_counters",
            MagicMock(return_value=STUB_NET_IO_COUNTERS),
        ):
            config = [
                {
                    "interfaces": {
                        "eth0": {
                            "type": "greater",
                            "bytes_sent": 100000,
                            "bytes_recv": 100000,
                            "packets_sent": 100000,
                            "packets_recv": 100000,
                            "errin": 0,
                            "errout": 0,
                            "dropin": 0,
                            "dropout": 0,
                        }
                    }
                }
            ]

            ret = network_info.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            _expected_return = [
                {
                    "interface": "eth0",
                    "network_info": {
                        "bytes_recv": 914626664,
                        "bytes_sent": 93662618,
                        "dropin": 0,
                        "dropout": 0,
                        "errin": 0,
                        "errout": 0,
                        "packets_recv": 903802,
                        "packets_sent": 465694,
                    },
                }
            ]

            ret = network_info.beacon(config)
            self.assertEqual(ret, _expected_return)
